// |$Id: bobf.c,v 1.1.1.1 2024/10/07 17:24:38 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://173.255.241.228              | 
   | FUNCTION NAME: bobf.c                                      |
   | DIRECTORY: bsu-3.0.3/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: computes the angles between seismic traces        |
   |          in two different bsegy files.                     |
   |                                                            |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2021 Paul Michaels                          |
   |  <paulmichaels@boisestate.edu>                             |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
        char infile1[], char infile2[], float *tmin, float *tmax,
        char lstfile[], char pid[]);

      void outlst(char infile1[], char infile2[],
        char pid[], char lstfile[], FILE *h3);

	float angle(float *s1, float *s2, int jstart, int jend);

	void saveplot (FILE * h5 ,int ntraces, float Deg[], float RANGE[], float meanAngle,
	float stdevAngle, char infile1[], char infile2[], float OBJF);


      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1,  *h3, *h4, *h5;
      int               jt,
                        jrec,
                        npts, npts4,
                        ntraces, ntraces4,
                        ibar,
                        invbar;
      float             fsamin, fsamin4,
                        bar,
                        *s1,
                        *s2,
			*Deg;
      float		sx,sy,sz,sd,rx,ry,rz,range;
      float		sx4,sy4,sz4,sd4,rx4,ry4,rz4,range4;
      float		*RANGE;
      float		tmin,
			tmax;
      char              pid[5]="bobf",   //process id name (4char)
                        lstfile[13],
                        infile1[14],
                        infile2[14];
      struct header     hd,
			hd4;

//...specific to this application
      	int             jstart;
	int             jend;  
	float		meanAngle;
	float		stdevAngle;
	float		DELT;
	float		FMAX1, FMIN1;
	float		FMAX2, FMIN2;
	int		isw1 = 1; //switch to find max and min absolute values fmax_min function
	int		KMX1, KMN1;
	int		KMX2, KMN2;
	float		*kmaxdif;
	float		meanKmaxdif;
	float		OBJF;



//...get parameters
      if(getparm(argc,argv,infile1,infile2,&tmin,&tmax,
         lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(infile1,infile2,pid,lstfile,h3);

//...open input files
        if( (h1=fopen(infile1,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile1);
          goto exit_3;
        }
        if( (h4=fopen(infile2,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile2);
          goto exit_3;
        }

          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }
          if(in_chk(&ntraces4,&npts4,&fsamin4,hd,h4)!=0)
         {
          goto exit_2;
         }

//...check parameters
	if (fsamin != fsamin4) {
	fprintf(stderr,"ABORT: sample intervals not the same!! %.5f != %.5f \n",
	fsamin,fsamin4);
	goto exit_2;
	}
	jstart = (int) (tmin/fsamin);
	jend   = (int) (tmax/fsamin);
	if (jstart < 0 || jstart >jend  ) {
	jstart=0;
	fprintf(h3,"\n Reset jstart = %d ",jstart);
	}
	if (jend < 0 || jend < jstart  ) {
		if (npts <= npts4 ) jend = npts;
		if (npts4 < npts ) jend = npts4;
	fprintf(h3,"\n Reset jend = %d ",jend);
	}
	if (ntraces != ntraces4 ) {
	fprintf(stderr,"ABORT: number of traces not equal %d != %d \n",ntraces,ntraces4);
	goto exit_2;
	}

//...print input file data to lstfile
        fprintf(h3,"\n  (1)Number of samples= %d",npts);
        fprintf(h3,"\n  (1)Sample Interval = %6.5f sec.",fsamin);
        fprintf(h3,"\n  (1)Number of traces = %d",ntraces);
        fprintf(h3,"\n     Tmin = %f jstart = %d",tmin,jstart);
        fprintf(h3,"\n     tmax = %f jend   = %d \n",tmax,jend);
//...print input file data to lstfile
        fprintf(h3,"\n  (2)Number of samples= %d",npts4);
        fprintf(h3,"\n  (2)Sample Interval = %6.5f sec.",fsamin4);
        fprintf(h3,"\n  (2)Number of traces = %d \n",ntraces4);

fflush(h3);
//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
        s2=calloc(npts4,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts4);
          goto exit_s1;
         }

//...dynamic allocation of angle vector
	Deg=calloc(ntraces,sizeof(float));
	  if(Deg==NULL)
         {
          fprintf(stderr,"ABORT:Deg can't allocate enough angle memory");
          fprintf(stderr," ntraces = %d",ntraces);
          goto exit_s1;
         }
	RANGE=calloc(ntraces,sizeof(float));
	  if(RANGE==NULL)
         {
          fprintf(stderr,"ABORT:Deg can't allocate enough RANGE memory");
          fprintf(stderr," ntraces = %d",ntraces);
          goto exit_s1;
         }

//...allocate saved function of maximum arrival sample number differences 
	kmaxdif=calloc(ntraces,sizeof(float));
	if (kmaxdif==NULL)
	{
          fprintf(stderr,"ABORT:kmaxdif can't allocate enough memory");
	  goto exit_s1;
	} //endif

//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop
	//...print header
	fprintf(h3," Trace  Angle Arrival Dif \n");
	fprintf(h3,"        (deg)  (samples)\n");
	meanAngle = 0.00;
        meanKmaxdif = 0.00;

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read traces
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }
        if(c_bsegin(jrec,s2,npts4,&hd4,h4)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...get header info
	findxyz(&sx,&sy,&sz,&sd,&rx,&ry,&rz,&range,hd);
	findxyz(&sx4,&sy4,&sz4,&sd4,&rx4,&ry4,&rz4,&range4,hd4);
	if (range != range4) {
	fprintf(stderr,"ABORT: ranges don't match \n");
	abort();
	} //endif
	RANGE[jrec]=range;


//...find difference in maximum amplitude arrival times sample number
	fmax_min(npts,s1,&FMAX1,&FMIN1,isw1,&KMX1,&KMN1);
	fmax_min(npts,s2,&FMAX2,&FMIN2,isw1,&KMX2,&KMN2);
	kmaxdif[jrec] = fabs((float) KMX1 - (float) KMX2);
	meanKmaxdif = meanKmaxdif + kmaxdif[jrec];

//...print out to list file the angle and arrival time differences (samples)
	Deg[jrec] = angle(s1,s2,jstart,jend);
//...compute mean angle
	meanAngle = meanAngle + Deg[jrec];

	fprintf(h3," %d   \t%4.2f \t%.2f \n",jrec,Deg[jrec],kmaxdif[jrec]);


//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

	meanAngle = meanAngle/((float) ntraces);
	meanKmaxdif = meanKmaxdif/((float) ntraces);

//...compute standard deviation about mean
	stdevAngle = 0.00;
//	fprintf(stderr,"\n#Trace   Angle\n");
//	fprintf(stderr,"#        (deg)\n");
	for (jrec=0; jrec<ntraces; jrec++) {
	DELT=(meanAngle-Deg[jrec]);
//	fprintf(stderr," %5.5d   %4.2f \n",jrec,Deg[jrec]);
	stdevAngle = stdevAngle + DELT*DELT;
	} //next jrec
	stdevAngle = sqrt(stdevAngle/((float) ntraces));

//...compute objective function
	OBJF = stdevAngle + meanKmaxdif;

//	fprintf(stderr,"\n#\t \t Mean Angle = %.2f StdDev = %.2f \n",meanAngle,stdevAngle);
	fprintf(h3,"\t \t Mean Angle = %.2f StdDev = %.2f \n",meanAngle,stdevAngle);
	fprintf(h3,"\t \tMean Arrival Difference %.2f \n",meanKmaxdif);
	fprintf(h3,"\t \tObjective Function = %.2f \n",OBJF);
	fprintf(stdout,"\n\t \t Mean Angle = %.2f StdDev = %.2f \n",meanAngle,stdevAngle);
	fprintf(stdout,"\t \tMean Arrival Difference %.2f \n",meanKmaxdif);
	fprintf(stdout,"\t \tObjective Function = %.2f \n",OBJF);
//	fprintf(stderr,"%s %s %.2f  %.2f  \n",infile1,infile2,meanAngle,stdevAngle);
	fprintf(h3,"------------------------------------------------------\n");

//...Generate a gnuplot plot file
//...open output file
	char	ofile[13];
	sprintf(ofile,"graph.gp");
        if( (h5=fopen(ofile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

	saveplot(h5,ntraces,Deg,RANGE,meanAngle,stdevAngle,infile1,infile2,OBJF);
	fclose(h5); //need to close for system command to work
	system("gnuplot -p graph.gp\n");


      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Plot File ===>%s",ofile);
      printf("\n   Processed %d traces \n",ntraces);
	free(s1);
	free(s2);
        return(0);

quit_it:
        puts(" ");
        free(s2);
exit_s1:
        free(s1);
	free(Deg);
	free(kmaxdif);
exit_2:
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);
quick_exit:
        return(1);
     } //  end of main
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         char infile1[],char infile2[],float *tmin, float *tmax,
         char lstfile[],char pid[])
     {
      char              sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Compute Objective Function                   ");
      sprintf(msg2,"Small value indicates shot gathers match     ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile1  infile2  tmin  tmax  \n",pid);
      puts(" ");
      puts("     infile1  = input file1 name (4char minimum)");
      puts("     infile2  = input file2 name (4char minimum)");
      puts("     tmin = minimum time in seconds ");
      puts("     tmax = maximum time in seconds ");
      puts(" ");
      puts("           NOTE: Files must have same: ");
      puts("                 =>   traces   ");
      puts("                 =>   samples ");
      puts("                 =>   sample interval ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters

//...file 1
      if(argc>1)
       strncpy(infile1,argv[1],13);
       else
      {
       puts(" Enter INPUT file1 name ");
       scanf(" %13s",infile1);
      }
	if (namecheck(12,infile1) != 0 ) return (1);

//...file 2
      if(argc>2)
       strncpy(infile2,argv[2],13);
       else
      {
       puts(" Enter INPUT file2 name ");
       scanf(" %13s",infile2);
      }
	if (namecheck(12,infile2) != 0 ) return (1);

//...convert string to float  
      if(argc>3)
       *tmin=atof(argv[3]);
       else
      {
       puts(" Enter tmin ");
       scanf(" %f",tmin);
      }
//...convert string to float  
      if(argc>4)
       *tmax=atof(argv[4]);
       else
      {
       puts(" Enter tmax ");
       scanf(" %f",tmax);
      }

//...name output files
      names(infile1,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(char infile1[],char infile2[],
        char pid[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File1= %s",infile1);
      fprintf(h3,"\n     Input   File2= %s",infile2);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: angle.c                                     |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: compute angle between traces in specified         |
   |          time window.                                      |
   |                                                            |
   |------------------------------------------------------------|
*/

	float angle(float *s1, float *s2, int jstart, int jend)
	{
	float norm1, norm2;
	int	j;
	float iproduct;
	float A;
	float rad2deg = 180.0/3.141592653589793;

	norm1=0.;
	norm2=0.;
	iproduct=0;
		for (j = jstart; j<jend; j++)
		{
		norm1=norm1 + s1[j]*s1[j];
		norm2=norm2 + s2[j]*s2[j];
		iproduct = iproduct + s1[j]*s2[j];
		} //next j
	norm1 = sqrtf(norm1);
	norm2 = sqrtf(norm2);
	A = iproduct/(norm1*norm2);
	if (A > 1.0000000000000000 ) A = 1.00000;
	if (A < -1.0000000000000000 ) A = -1.00000;
	return(acosf(A)*rad2deg);
	} //end angle
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.boisestate.edu>|
   | FUNCTION NAME: genplot                                     |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: saves plot to a file                              |
   |                                                            |
   |------------------------------------------------------------|
*/
	void saveplot (FILE * h5 ,int ntraces, float Deg[], float RANGE[], float meanAngle,
	float stdevAngle, char infile1[], char infile2[], float OBJF) 
	{
	char	XL[13]="Range (m)";
	char	YL[]="Angle Between Traces (deg)";
	char	title[60];
	sprintf(title,"Files:%s %s [mean=%.1f std=(+/-) %.2f] OBJF=%.2f",
	infile1,infile2,meanAngle,stdevAngle,OBJF);
  	#ifdef WIN
  	char	outfile[]="graph.jpeg";
  	#else
  	char	outfile[]="graph.ps";
  	#endif
	float ybase=0.0;
  	fprintf(h5,"# Generated by bobf.c saveplot() function \n");
	fprintf(h5,"# Default is live plot \n");
	fprintf(h5,"# For fixed image file remove # on TYPE=image \n");
	fprintf(h5,"# and rerun:  gnuplot graph.gp \n");
  	fprintf(h5,"set grid \n");
#ifdef WIN
  	fprintf(h5,"TYPE=\"live\" \n"); 
  	fprintf(h5,"#TYPE=\"image\" \n"); 
  	fprintf(h5,"if (TYPE ne \"live\") { \n"); 
  	fprintf(h5,"set terminal jpeg large size 600,300 enhanced \n"); 
  	fprintf(h5,"set output \"%s\" \n",outfile); 
#else
  	fprintf(h5,"TYPE=\"live\" \n"); 
  	fprintf(h5,"#TYPE=\"image\" \n"); 
  	fprintf(h5,"if (TYPE ne \"live\") { \n"); 
  	fprintf(h5,"set terminal postscript size 6.0,3.0 in color \\\n"); 
	fprintf(h5,"portrait enhanced font \'Helvetica,10\' \n");
  	fprintf(h5,"set output \"%s\" \n",outfile); 
#endif
  	fprintf(h5," } else { \n"); 
	fprintf(h5,"TRM=GPVAL_TERM                   \n");
	fprintf(h5,"if (TRM eq \"unknown\" ) {set terminal x11 persist} \n");
	fprintf(h5,"if (TRM eq \"qt\") { set terminal wxt size 800,400 persist  } \n");
	fprintf(h5,"if (TRM eq \"wxt\") { set terminal wxt size 800,400 persist }   \n");
	fprintf(h5,"if (TRM eq \"x11\") { set terminal x11 persist } \n");
  	fprintf(h5," }  \n"); 
	fprintf(h5,"                                 \n");
  	fprintf(h5,"set key right \n");
  	fprintf(h5,"set xlabel \" %s \" \n",XL);
  	fprintf(h5,"set ylabel \" %s \" \n",YL);
//  	fprintf(h5,"set title \" %s \"  \n",infile);
        fprintf(h5,"set xrange [%f:%f] \n",RANGE[0],RANGE[ntraces-1]);

	float fmax,fmin;
	int   kmx,kmn;
	fmax_min(ntraces,Deg,&fmax,&fmin,0,&kmx,&kmn);

	if ((floorf(fmin)-stdevAngle) == (ceilf(fmax)+stdevAngle)) {
        fprintf(h5,"set yrange [%f:%f] \n",0.0,180.0);
	} else {
        fprintf(h5,"set yrange [%f:%f] \n",floorf(fmin)-stdevAngle,ceilf(fmax)+stdevAngle);
	} //endif
  	fprintf(h5,"set title \"%s \" \n",title);
	fprintf(h5,"mean(x)=%f \n",meanAngle);
	fprintf(h5,"stdp(x)=mean(x)+%f \n",stdevAngle);
	fprintf(h5,"stdm(x)=mean(x)-%f \n",stdevAngle);

  	fprintf(h5," plot '-'  u ($1):($2) lc 'black' w l  t \"angle (deg.) \",\\");
	fprintf(h5,"\n");
	fprintf(h5," '-' u ($1):($2) w p pt 6  t \" \" , mean(x) lc 'red' w l t \"mean\",\\");
	fprintf(h5,"\n");
        fprintf(h5," stdp(x) lc 'blue' w l t \"std\", stdm(x) lc 'blue' w l t \" \" \n");
  	int j;
  		for(j=0;j<ntraces;j++) {
    		fprintf(h5,"%f %e \n",RANGE[j],Deg[j]);
  		} //next j 
  	fprintf(h5,"e\n");  //this is required when std in - plot

  		for(j=0;j<ntraces;j++) {
    		fprintf(h5,"%f %e \n",RANGE[j],Deg[j]);
  		} //next j 
  	fprintf(h5,"e\n");  //this is required when std in - plot

	} //end saveplot

