// |$Id: bxof.c,v 1.1.1.1 2024/10/07 17:24:38 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://173.255.241.228              | 
   | FUNCTION NAME: bxof.c                                      |
   | DIRECTORY: bsu-3.0.3/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: master example of bsegy c-language                |
   |          seismic processing program (takes absolute values)|
   |          Analogous to bmst.f (Fortran version)             |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2021 Paul Michaels                          |
   |  <paulmichaels@boisestate.edu>                             |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

	float  sgn(float delt); 

	int  getparm(int argc,char *argv[],
	float *near, float *far, char infile[],
        char ofile[], char lstfile[], char pid[]);

	void outlst(float near, float far, char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2;
      char              pid[5]="bxof",   //process id name (4char)
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
	float		near;
	float		far;
	float		sx,
			sy,
			sz,
			sd,
			rx,
			ry,
			rz,
			frange;
	float		offset;
	int		krec;
	float		testn,
			testf;
	float		delx,
			dely;
			
//...get parameters
      if(getparm(argc,argv,&near,
         &far,infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(near,far,infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }


//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d \n",ntraces);
fflush(h3);
//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//deal with possible zero offset case
	jrec=0;
//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }
//...get trace geometry
float	sx0,sy0,sz0,rx0,ry0,rz0;
	findxyz(&sx0,&sy0,&sz0,&sd,&rx0,&ry0,&rz0,&frange,hd);
	jrec=1;
//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }
//...get trace geometry
float	sx1,sy1,sz1,rx1,ry1,rz1;
	findxyz(&sx1,&sy1,&sz1,&sd,&rx1,&ry1,&rz1,&frange,hd);

//...define signed offset for test
	delx = rx1-rx0;
	dely = ry1-ry0;




//...trace loop
	krec = 0;

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...get trace geometry
	findxyz(&sx,&sy,&sz,&sd,&rx,&ry,&rz,&frange,hd);

	if (fabs(delx) > fabs(dely)) {
	offset = sgn(delx)*frange;
	} else {
	offset = sgn(dely)*frange;
	} //endif

	if (offset < 0 ) {
		testn = far;
		testf = near;
	} else {
		testn = near;
		testf = far;
	} //endif

	if ( (offset <= testf) && (offset >= testn)   ) {

//...output trace
        if(c_bsegout(krec,s1,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }
	fprintf(h3,"Selected: offset = %f  trace = %d  SP %4.4s REC %4.4s \n",
	offset,jrec,hd.spname,hd.rstnam);	
	krec = krec +1 ; 

	} //endif

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces \n",ntraces);
      printf("   Selected %d traces \n",krec);
	free(s1);
	free(s2);
        return(0);

quit_it:
        puts(" ");
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);
quick_exit:
        return(1);
     } //  end of main
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: sgn.c                                       |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: sign function, returns sign of delt               |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
	float	sgn(float delt) {
		return(delt/fabs(delt));
	}


/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         float *near, float *far,char infile[],
         char ofile[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"BXOF: Extract traces by offset.              ");
      sprintf(msg2,"Off end and split spreads by offset sign.    ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile   near  far  \n",pid);
      puts(" ");
      puts("     infile  = input file name (4char minimum)");
      puts(" ");
      puts("     near    = near offset (float) to output  ");
      puts("     far     = far  offset (float) to output  ");
      puts(" ");
      puts("     Offset values signed to handle split spreads. ");
      puts("     Negative offsets one side of shot, Positive the other.");
      puts("     NOTE: !! When using negative offsets:  ");
      puts("     near is the offset with the smallest absolute value");
      puts("     far  is the offset with the largest absolute value");
      puts("     See man page for definition of offsets. ");
      puts("     To select both sides of a split spread requires 2 runs ");
      puts("     and concatenation of the two runs into a single file. ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//... near offset
      if(argc>2)
       *near = atof(argv[2]);
       else
      {
       puts(" Enter near offset (signed float)");
       scanf(" %f",near);
      }

//...convert string to float
      if(argc>3)
       *far = atof(argv[3]);
       else
      {
       puts(" Enter far offset  (signed float)");
       scanf(" %f",far);
      }

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(float near, float far,char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     near offset = %7.3f ",near);
      fprintf(h3,"\n     far offset  = %7.3f ",far);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
