// |$Id: genbzrt.c,v 1.1.1.1 2024/10/07 17:24:38 pm Exp $           |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P.MICHAELS <paulmichaels@boisestate.boisestate.edu>|
   | FUNCTION NAME: genplot                                     |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose:  Generate A Sequence of bzrt runs                 |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>

int 	main(int argc, char *argv[]) {
	FILE	*h1, *h2;
	char	outfile[]="gobzrt";
	char	gpfile[]="VPCA.gp";
	char	infile[20];
	char	infileshort[13];
	char	tail[]=".seg";
	float	tend;
	float	tstart;
	float	tstep;
	float	tgate;
	float	tlagmx;
	int	j, nstep;
	float	t1, t2;
	int	chV;
	int	chH;
	int	ipct;
	int  	hsw1;
	#ifdef WIN
	char Xwin[4]="wxt";
	#else
	char Xwin[4]="x11";
	#endif
	if(argc<10 ) {
	puts("USAGE: genbzrt filename tstart tend tstep tgate chV chH ipct hsw1  \n");
	puts("filename = input file name");
	puts("tstart   = start time (float, seconds) of processing");
	puts("tend     = end time (float, seconds) of processing)");
	puts("tstep    = time shift forward with sliding window (float, seconds) of processing)");
	puts("tgate    = length in time of sliding window (float, seconds)  ");
	puts("chV      = channel with V-component (int)                                        ");
	puts("chH      = channel with H-component (int)                                        ");
	puts("ipct     = percent of max amplitude to include in window (int)                   ");
	puts("hsw1     = switch to set Horizontal Component ID                                 ");
	puts("           Example:  hsw1=2    IF Horizontal component is labeled R-component    ");
	puts("                     hsw1=3    IF Horizontal component is labeled T-component    ");
	puts("          IMPORTANT:  Make sure that hsw1 and chH are in agreement               ");
	puts("\n ABORT");
	fprintf(stderr," you only have %d arguments on command line \n",argc);
	for (j=0; j<argc; j++) fprintf(stderr,"argv[%d] = %s \n",j,argv[j]);
	return(1);
	}
	else {
	strncpy(infile,argv[1],19);
	tstart = atof(argv[2]);
	tend = atof(argv[3]);
	tstep = atof(argv[4]);
	tgate = atof(argv[5]);
	chV   = atoi(argv[6]);
	chH   = atoi(argv[7]);
	ipct  = atoi(argv[8]);
	hsw1  = atoi(argv[9]);
	} //endif

//...Clean Files
//	int ret = system("rm -f bzrtpca.dat");
//	puts("cleaning pca list");
//	fprintf(stdout,"ret= %d \n",ret);


//open output file for writing
        if( (h1=fopen(outfile,"w"))==NULL )
        {
        printf("\n ABORT !! Error opening file %s \n",outfile);
	fflush(stderr);
	  abort();
        }
	fprintf(h1,"#!/bin/bash \n");

//...echocheck
	fprintf(h1,"# GENERATED BY GENBZRT.C          \n");
	fprintf(h1,"# (Vertical Plane Motion)         \n");
	fprintf(h1,"# ================================\n");
	fprintf(h1,"# Input File          = %s \n",infile);
	fprintf(h1,"# Start Time          = %.2f sec. \n",tstart);
	fprintf(h1,"# End Time            = %.2f sec. \n",tend);
	fprintf(h1,"# Time Step (moveups) = %.2f sec. \n",tstep);
	fprintf(h1,"# Time Gate           = %.2f sec. \n",tgate);
	fprintf(h1,"# chV                 = %d        \n",chV);
	fprintf(h1,"# chH                 = %d        \n",chH);
	fprintf(h1,"# ipct                = %d        \n",ipct);
	fprintf(h1,"# hsw1                = %d        \n",hsw1);
	fprintf(h1,"#  \n");

//...clean pre-existing VPCA output
	fprintf(h1,"rm -f bzrtpca.dat \n");
	fprintf(h1,"rm -f bzrthodo.ps \n");

// Generate loop of commands
	nstep = (tend - tstart)/tstep -1 ;
	t1=tstart;
	t2=t1 + tgate;
	fprintf(h1,"echo ""gate: %.1f %.1f "" \n ",t1,t2);
	fprintf(h1,"bzrt %s %d %d %i %d %f %f >/dev/null \n",infile,chV,chH,ipct,hsw1,t1,t2);
	fprintf(h1,"cat bzrt.ps >> bzrthodo.ps \n");
	t1 = t1 + tstep;
	for (j=0; j<nstep-1; j++) {
	t2=t1 + tgate;
	fprintf(h1,"echo ""gate: %.1f %.1f "" \n ",t1,t2);
	fprintf(h1,"bzrt %s %d %d %i %d %f %f >/dev/null \n",infile,chV,chH,ipct,hsw1,t1,t2);
	t1 = t1 + tstep; 	
	fprintf(h1,"cat bzrt.ps >> bzrthodo.ps \n");
	} //next j
	fprintf(h1,"ps2pdf bzrthodo.ps \n");
	fprintf(h1,"gnuplot VPCA.gp \n");

//open gnuplot file for writing
        if( (h2=fopen(gpfile,"w"))==NULL )
        {
        printf("\n ABORT !! Error opening file %s \n",gpfile);
	fflush(stderr);
	  abort();
        }

	fprintf(h2,"# GENERATED BY GENBZRT.C  \n");
	fprintf(h2,"# Channels: V=%d  H=%d \n",chV,chH);
	fprintf(h2,"set grid    \n");
	fprintf(h2,"set terminal %s persist \n",Xwin);
	fprintf(h2,"set key left    \n");
	fprintf(h2,"set xlabel 'Time (s)'    \n");
        fprintf(h2,"set ylabel 'Angle wrt V (deg) and Normalized Amp.' \n");
	fprintf(h2,"set title 'Vertical Motion %s V=%d H=%d' \n",infile,chV,chH);
	fprintf(h2,"stats 'bzrtpca.dat' u 4    \n");
	fprintf(h2,"p 'bzrtpca.dat' u 1:3 w l lc 'black' lw 2 t 'angle wrt V',\\");
	fprintf(h2,"\n 'bzrtpca.dat' u 1:($4/STATS_max*200 ) \\");
	fprintf(h2,"\n w p pt 6 ps 1.5 lc 'blue'  t 'normalized amp'  \n");
	fprintf(h2,"set terminal postscript enhanced color   \n");
	fprintf(h2,"set output 'VPCA.ps'    \n");
	fprintf(h2,"replot    \n");
   



//...cleanup
	fprintf(h1,"#rm -f bzrt.ps \n");
	fprintf(h1,"#rm -f bzrthodo.ps \n");

	fclose(h1);
	fclose(h2);
	system("chmod +x gobzrt");
	puts("SUCESS Now you run the output script");
	printf("Output Bash Script ==> gobzrt \n");
	printf("Script will run Gnuplot script after gobzrt ==>VPCA.gp ==>VPCA.ps  \n"); 
} //endmain

