// |$Id: sac2seg.c,v 1.1.1.1 2024/10/07 17:24:38 pm Exp $           |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://173.255.241.228              | 
   | FUNCTION NAME: sac2seg.c                                   |
   | DIRECTORY: bsu-3.0.3/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: convert SAC   format data (Iris typical)          |
   |          to BSEGY formatted data (limited by number of     |
   |          samples in a BSEGY header                         |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2021 Paul Michaels                          |
   |  <paulmichaels@boisestate.edu>                             |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
// Paul Michaels <paulmichaels@boisestate.edu>
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/bsac.h"
      #include "../include/c_bsegy.h"
      #include "../include/sub4.h"

//...Original Code from bsac-1.0.0 just had bsac.h include
// compile command: gcc -Wall -lm sac2seg.c -o sac2seg
//reads a evenly spaced SAC file and outputs to stdout
// Two Columns: sample_time data_value 

//prototypes
        int  getparm(int argc,char *argv[], char infile[], char pid[]);

	void names2(char filename[],char Station[],
	char channel[], char sfx[]);

	void  genhead(int jrec, float fsamin, int npts, float samprate, char channel[],
	char Station[], 
        int year, int day, int hour, int min, int sec,
        struct header *hd);

	void	gengnuplot(char plotfile[], char Station[], char channel[],
	int year, int day, int hour, int min, int sec,
	char datfile[]);

	int	findnptshort(int npts);
	

	short doy2moday(int doy,  int year);

	int	GetHeader(char infile[],int NZDTTM[], FILE *h3, struct sacheader *hdsac);

	void	DisplayHeader(struct sacheader hdsac, int NZDTTM[],
		 double *tzero, int *npts, float *delt, FILE *h3 );

	int	GetSacData(int npts, char infile[], float *s1);

      int  main(int argc, char *argv[])
     {

//...declare variables
     	FILE            *h1,	
			*h2,
			*h3,
			*h4;
      	int             j;
      	float           *s1; 
	float		*s2;
	double  	tm;
	float		delt;
	char		plotfile[13],
                        ofile[13],
			datfile[13],
			lstfile[13];
	char		pid[5]="sac2";
	char		sfx1[4]="lst";
	char		sfx2[4]="dat";
	char		sfx3[4]="seg";
	int		jrec;
	int		ntraces;
	double		tzero;  //time zero GMT Hr Min Sec msec==>seconds
	int		npts;
	int		NZDTTM[6]={0,0,0,0,0,0};
	char		infile[132];   //OK SAC files likely to have a long name
                                       // so making an exception like with mseed2seg.c
	struct sacheader hdsac;
	struct header	hd;

//...get input parameters
        if (getparm(argc,argv,infile,pid) != 0)
	{
	goto quick_exit;
	}

	/* Get SAC Header for Input File */
	if (GetHeader(infile,NZDTTM,h3,&hdsac) == 1 ) goto exit_1;

	/* Name listing file  */
	char Station[11];
	for (j=0; j<11; j++) Station[j]='\0';
	strncpy(Station,hdsac.KSTNM,7);
	for (j=0; j<11; j++) if (Station[j]==' ') Station[j]='\0';
	char channel[11];
	for (j=0; j<11; j++) channel[j]='\0';
	strncpy(channel,hdsac.KCMPNM,7);
	for (j=0; j<11; j++) if (channel[j]==' ') channel[j]='\0';
	names2(lstfile,Station,channel,sfx1);
	names2(datfile,Station,channel,sfx2);
	names2(ofile,Station,channel,sfx3);

//...open output file
          if( (h4=fopen(ofile,"wb"))==NULL )
          {
            fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
            goto exit_1;
          }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(h3,"\n ABORT !! Error opening file %s",lstfile);
          return(1);
        }

	fprintf(h3," Program sac2seg \n Infile: %s \n",infile);

	/* Display Header, extract some values */
	DisplayHeader(hdsac,NZDTTM,&tzero,&npts,&delt,h3);
	float fsamin = delt;  //sample interval float

	/* Check for evenly spaced data */
	if (hdsac.LEVEN != 1 ) {
	fprintf(h3,"ABORT!!!! Data not evenly spaced \n");
	return(1);} //endif


//...dynamic allocation of memory for data
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(h3,"ABORT:s1 can't allocate enough trace memory");
          fprintf(h3," npts  = %d",npts);
          goto exit_s1;
         }

	/* Read Data from Input File */
	int	rcode;
	if ((rcode=GetSacData(npts,infile,s1)) != 0 ) {
	 fprintf(h3," ERROR: reading data rcode=%d\n",rcode);
	goto exit_1; } //endif
	fflush(stderr);

//...open datfile
        if( (h2=fopen(datfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",datfile);
          goto quick_exit;
        }

	for (j=0; j<npts; j++) {
//	tm=(float) j * delt;  //for first sample = time zero
//	tm=(double) j * (double) delt + tzero;  //for seconds into day GMT
	tm=(double) j * (double) delt;
	fprintf(h2,"%lf %.1f \n",tm,s1[j]);
	}

//...Compute output traces length for BSEGY
	int nptshort = 65535;
	if (npts > nptshort ) {
	nptshort = findnptshort(npts);
	if (nptshort == 0) nptshort = 65535; // if failed to find divisor <1000
	ntraces = npts / nptshort;
	int remain = npts%nptshort;
   	fprintf(h3,"==> Number of traces = %d \n",ntraces);
		if (remain != 0 ) {
		fprintf(stdout,"|   Residual data not output (modulo test) \n");
		fprintf(stdout,"|   Remainder = %d \n",remain);
  		fprintf(h3,"==> Residual data not output (modulo test) \n");
  		fprintf(h3,"==> Remainder = %d \n",remain);
		}
	}
	else {
	nptshort = npts;
	ntraces = 1;
	} //endif test for large trace

//...output traces
	float samprate = 1.0/fsamin;
	int year = NZDTTM[0];
	int day  = NZDTTM[1];
	int hour = NZDTTM[2];
	int min  = NZDTTM[3];
	int sec  = NZDTTM[4];
	float	*s1ptr = &s1[0];

//...output loop BSEGY
	for (jrec=0; jrec <ntraces; jrec++) { 
//...generate a BSEGY header
	genhead(jrec,fsamin,nptshort,samprate,channel,Station,
	year,day,hour,min,sec,&hd); 

        if(c_bsegout(jrec,s1ptr,nptshort,&hd,h4)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }
	s1ptr = s1ptr + nptshort;
	} //next jrec

//...generate a Gnuplot file for datfile
	gengnuplot(plotfile,Station,channel,year,day,hour,min,sec,datfile);

//... alternative test for long names
	if (strlen(infile)>12) {
	puts("|   Warning: long file name, check output file name for truncations");
	fprintf(stdout,"|   InputFile: %s \n",infile);
	}
	puts("------------------------Output Files---------------------------------");
	printf(" *.lst file ====>%s \n",lstfile);
	printf("BSEGY *.seg file  ====>%s \n",ofile);
	printf("ASCII *.dat file ====>%s \n",datfile);
	printf("Gnuplot *.gp  file ====>%s \n",plotfile);
        printf("   Number of traces = %d \n",ntraces);

quit_it:
	free(s2);

exit_1:
        fclose(h2);
  	fclose(h3);
exit_s1:
        free(s1);
return(rcode);

quick_exit:
	return(1);
}	//endmain
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: GetHeader                                   |
   | Copyright (C) 2022 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: Read SAC header from infile                       |
   |          Output to listing file *h3                        |
   |------------------------------------------------------------|
*/
	int	GetHeader(char infile[], int NZDTTM[], FILE *h3, struct sacheader *hdsac)
	{
	FILE	*h1;
//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(h3,"\n ABORT !! Error opening file %s",infile);
          return(1);
        }

//...read header
        if(fread(hdsac,sizeof(char),158*4,h1)!=158*4)
            {
             puts("ABORT--Error reading input file ");
             return(1);
            }
	NZDTTM[0]=(*hdsac).NZYEAR;
	NZDTTM[1]=(*hdsac).NZJDAY;
	NZDTTM[2]=(*hdsac).NZHOUR;
	NZDTTM[3]=(*hdsac).NZMIN;
	NZDTTM[4]=(*hdsac).NZSEC;
	NZDTTM[5]=(*hdsac).NZMSEC;
	fclose(h1);
	return(0);
	} //end GetHeader
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME:  DisplayHeader                              |
   | Copyright (C) 2022 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: Read SAC header information from infile           |
   |                                                            |
   |------------------------------------------------------------|
*/
	void	DisplayHeader(struct sacheader hdsac, int NZDTTM[],
	 double *tzero, int *npts, float *delt, FILE *h3 )
	{
	/*  Station  */
	char	KSTNM[8]="       \0";
	strncpy(KSTNM,hdsac.KSTNM,7);
	fprintf(h3,"\n Station Name: %s \n",KSTNM);
	fprintf(h3," Station Latitude = %f degrees (+North)\n",hdsac.STLA);
	fprintf(h3," Station Longitude= %f degrees (+East)\n",hdsac.STLO);
	fprintf(h3," Station Elevation= %f meters\n",hdsac.STEL);
	fprintf(h3," Station Below Surface Depth= %f meters\n",hdsac.EVDP);
	if (hdsac.BAZ  != -12345.0)
	fprintf(h3," Station to Event Azimuth = %f \n",hdsac.BAZ );
	if (hdsac.DIST != -12345.0)
	fprintf(h3," Station to Event Distance = %.2f km \n",hdsac.DIST);
	if (hdsac.GCARC != -12345.0)
	fprintf(h3," Station to Event Great Circle Arc %.2f deg \n",hdsac.GCARC);


	char	KCMPNM[8]="       \0";
	strncpy(KCMPNM,hdsac.KCMPNM,7);
	fprintf(h3," Channel: %s \n",KCMPNM);
	fprintf(h3,"------------------------------------------------\n\n");
	/* copy time date to array */
	fprintf(h3," ALL TIMES are GMT ");
	fprintf(h3,"\n Year=%d ",NZDTTM[0]);
	fprintf(h3," Julian Day=%d ",NZDTTM[1]);
	short moday;
	moday = doy2moday((int) NZDTTM[1], (int) NZDTTM[0]);
	int month = moday/100;
	int Day = moday-month*100;
	fprintf(h3,"\n DATE: %2.2d/%2.2d/%4.4d",month,Day,NZDTTM[0]);

	fprintf(h3,"\n Hour=%d ",NZDTTM[2]);
	fprintf(h3," Minute=%d ",NZDTTM[3]);
	fprintf(h3," Second=%d ",NZDTTM[4]);
	fprintf(h3," mSecond=%d ",NZDTTM[5]);

//...Calculate tzero
	*tzero= (double)(NZDTTM[1]*86400 + NZDTTM[2]*3600
	 + NZDTTM[3]*60 + NZDTTM[4] + ((double) NZDTTM[5])/1000.0 );
	fprintf(h3,"\n tzero=%f seconds  \n",*tzero);
	fprintf(h3,"------------------------------------------------\n\n");

//...Event headers
	if (hdsac.MAG != -12345.0)
	fprintf(h3," Event Magnitude = %f \n",hdsac.MAG);
	if (hdsac.EVLA != -12345.0)
	fprintf(h3," Event latitude  (degres, + North) = %f \n",hdsac.EVLA);
	if (hdsac.EVLO != -12345.0)
	fprintf(h3," Event longitude (degres, + East ) = %f \n",hdsac.EVLO);
	if (hdsac.EVEL != -12345.0)
	fprintf(h3," Event Elevation (meters)          = %f \n",hdsac.EVEL);
	if (hdsac.EVDP != -12345.0)
	fprintf(h3," Event Depth                       = %f \n",hdsac.EVDP);
	if (hdsac.AZ   != -12345.0)
	fprintf(h3," Event to Sation  Azimuth          = %f \n",hdsac.AZ  );
	fprintf(h3,"------------------------------------------------\n\n");

	/* Echo Check of key header values */	
	*npts=hdsac.NPTS;
	fprintf(h3," npts=%d \n",*npts);
	*delt=hdsac.DELTA;
	fprintf(h3," delt=%f \n",*delt);
	fprintf(h3," Begin Time: %f seconds\n",hdsac.B);
	fprintf(h3," End   Time: %f seconds\n",hdsac.E);
	fprintf(h3,"------------------------------------------------\n\n");
	
	/* Echo type of data */
	switch (hdsac.IFTYPE) {
		case 1:
		fprintf(h3," Type: time series data \n");
		break;
		case 2:
		fprintf(h3," Type: real and imag spectrum \n");
		break;
		case 3: 
		fprintf(h3," Type:spectral file, amplitude and phase \n");
		break;
		case 4:
		fprintf(h3," Type: general x vs y data \n");
		break;
		case 5:
		fprintf(h3," Type: general x,y,z (3-D) file \n");
		break;
	} //endswitch

	//fprintf(h3," SCALE=%f  \n",hdsac.SCALE);
		if (hdsac.IFTYPE != 1 ) {
		fprintf(stderr,"ALERT: data not a time series \n");
		fprintf(stderr,"Only *.dat file output \n");
		fflush(stderr);
		}//endif	

	switch (hdsac.IDEP) {
		case 5:
		fprintf(h3," X-data Type: UNKNOWN \n");
		break;
		case 6:
		fprintf(h3," X-data Type: displacement (nm) \n");
		break;
		case 7:
		fprintf(h3," X-data Type: velocity (nm/sec) \n");
		break;
		case 50:
		fprintf(h3," X-data Type: velocity signal in volts \n");
		break;
		case 8:
		fprintf(h3," X-data Type: accel (nm/sec/sec) \n");
		break;
	} //endswitch

	if (hdsac.DEPMEN != -12345.0) {
		fprintf(h3," Data Maximum = %f \n",hdsac.DEPMAX);
		fprintf(h3," Data Mean    = %f \n",hdsac.DEPMEN);
		fprintf(h3," Data Minimum = %f \n",hdsac.DEPMIN);
	} //endif

  	if (hdsac.IZTYPE != -12345.0) {
	switch (hdsac.IZTYPE) {
		case 5:
		fprintf(h3," Reference Time: UNKNOWN \n");
		break;
		case 9:
		fprintf(h3," Reference Time: Zero time is start of file \n");
		break;
		case 10:
		fprintf(h3," Reference Time: Zero time is 0000 of GMT day \n");
		break;
		case 11:
		fprintf(h3," Reference Time: Zero time is event origin \n");
		break;
		case 12:
		fprintf(h3," Reference Time: Zero time is 1st arrival \n");
		break;
		default:
		fprintf(h3," Reference Time:  IZTYPE=%d \n",
			hdsac.IZTYPE);
	} //endswitch
  	} //endif
	} // end DisplayHeader
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME:  GetSacData                                 |
   | Copyright (C) 2022 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: Read SAC binary data                              |
   |                                                            |
   |------------------------------------------------------------|
*/
	int	GetSacData(int npts, char infile[], float *s1)
	{
	FILE	*h1;
	
//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          return(1);  
        }
	
//move to end of header
  	fseek(h1,158*4,SEEK_SET);
	
//read data
   	if (fread(s1,sizeof(float),npts,h1)!=npts)
	{ puts("error reading data");
		return(1);
	}
//close input data file
	fclose(h1);
	return(0);
	} //end GetSacData
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: names2                                      |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: Output file name assumes 4 char station and       |
   |          3 char channel                                    |
   |          name generated ends in suffix.                    |
   |                                                            |
   |------------------------------------------------------------|
*/
	void	names2(char filename[],char Station[],
	 char channel[], char sfx[])
	{
	int	j;
	int 	lensta;
	for (j=0; j<13; j++) filename[j]='\0';
	lensta = strlen(Station);
	for (j=0;j<lensta;j++) filename[j]=Station[j];
	for (j=lensta;j<lensta+3;j++) filename[j]=channel[j-lensta];
	filename[lensta+3] = '.';
	for (j=lensta+4;j<(lensta+7);j++) filename[j]=sfx[j-(lensta+4)];
	}

//============================================================
/*
	Given day of year (doy), return an integer encoded
	as monthday.  For example moday = 0602 for June 02
*/
	short doy2moday(int doy,  int year)
	{
	int	days[12]={31,28,31,30,31,30,31,31,
				  30,31,30,31} ;
	int	j=0;
	int	sumL =0;
	int	sumU =0;
	int	month;
	int	dom;
	short	moday; // month.day

	if ((year % 4)==0  ) days[1]=29;


	while (j<12) {
	sumU = sumU + days[j];
	//...figure leap year
	if (sumU >= doy) break;
	j =j + 1;
	sumL = sumU;
	} // endwhile next j

//...found
	dom = doy - sumL;
	month = j+1;
	moday = month*100+dom;
	return(moday);
	} //end doy2moday
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2022 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[], char infile[], char pid[])
     {
      char		help[3]="-h";
      char              msg1[46],msg2[46];
//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Converts SAC format data to ASCII and BSEGY  ");
      sprintf(msg2,"Provides a Gnuplot file                      ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile    \n",pid);
      puts(" ");
      puts("     infile  = input file name (4char min, 131 char max");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],131);
       else
      {
       puts(" Enter INPUT file name (4 char min, 131 char max ");
       scanf(" %131s",infile);
      }
	if (namecheck(131,infile) != 0 ) return (1);

       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: genhead.c                                   |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose:  generate a 240 byte trace header                 |
   |                                                            |
   |------------------------------------------------------------|
*/
void  genhead(int jrec, float fsamin, int npts, float samprate, char channel[], 
	char Station[], 
        int year, int day, int hour, int min, int sec,
        struct header *hd)	
{

  int  jt;
  char buf1[10];
  short moday;
//generate trace header
      (*hd).tsnl = (long) jrec + 1;
      (*hd).tsnt = (long) jrec + 1;
      (*hd).ofrn = (long) 1;
      (*hd).tnofr = (long) jrec + 1;
      (*hd).cdp = (long) 1;
      (*hd).duse = (short) 1;
      (*hd).nvs = (short) 1;
      (*hd).nhs = (short) 1;
      (*hd).tic = (short) 1;
      (*hd).ictime = (short) 0;
      (*hd).isi =  (int) (fsamin * 1000000. + 0.5);
      (*hd).length = (int) npts;
      (*hd).gaint = (short) 3;
      (*hd).gidb = (short) 0;
      (*hd).utimes = (short) 0;
      (*hd).utimeg = (short) 0;
      (*hd).charge = (short) 0;
      (*hd).smul1 = (short) -1000;
      (*hd).smul2 = (short) -1000;
      for (jt = 0; jt < 4; jt++) {
	(*hd).spname[jt] = ' ';
	(*hd).rstnam[jt] = ' ';
	}
      (*hd).spname[3] = '1';

//      sprintf (buf1, "%4.4d", jrec + 1);

      for (jt = 0; jt < 3; jt++)
	(*hd).rstnam[jt+1] = channel[jt];
//	(*hd).rstnam[jt] = buf1[jt];

//      sprintf (buf1, "0001");
      for (jt = 0; jt < 4; jt++)
	(*hd).lineid[jt] = Station[jt];
//	(*hd).lineid[jt] = buf1[jt];
//...Because miniseed does not have a header for BH1 and BH2 azimuth, a guess
	if (channel[2]=='E' || channel[2]=='1') {
	(*hd).geoazi = (short) 90;}
	else {
	(*hd).geoazi = (short) 0;}

	if (channel[2]=='Z') {
	 (*hd).geover = (short) 180;}  //not sure of polarity convention, up or down
	else {
	(*hd).geover = (short) 90;} //horizontal component
      
      (*hd).shtver = (short) 0; //no way to know source moment tensor from miniseed
      (*hd).shtazi = (short) 0; 
      (*hd).mst = (long) 0;
      (*hd).sday = (short) 0;
      (*hd).shour = (short) 0;
      (*hd).smin = (short) 0;
      (*hd).sseco = (short) 0;
      (*hd).tyear = (short) year;
/*    (*hd).tday = (short)  day;   // day = day of year
      program bdump.f expects this format 4 digits mmdd
      This format was used by Bison Seismographs, 
      See egg2seg.c code also impacted by this original	
      format decision and coding of bdump.f
      Function doy2moday converts from day of year to mmdd format
*/
      moday = doy2moday((int) day, (int) year);
      (*hd).tday = (short)  moday;
      (*hd).thour = (short) hour ;
      (*hd).tmin = (short)  min;
      (*hd).tsec = (short)  sec;
      (*hd).ishlo = (long) 0;
      (*hd).ishla = (long) 0;
      (*hd).ishd = (long) 0;
      (*hd).ishe = (long) 0;
      (*hd).irlo = (long) 0;
      (*hd).irla = (long) 0;
      (*hd).irel = (long) 0;
      (*hd).flc = (short) 0;
      (*hd).fhc = (short) samprate/2; //miniseed does not have filters
      }  // end genhead
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: gengnuplot                                  |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: Generates a gnuplot script to plot *.dat outfile  |
   |                                                            |
   |------------------------------------------------------------|
*/
	void	gengnuplot(char plotfile[], char Station[], char channel[],
	int year, int day, int hour, int min, int sec,
	char datfile[])

	{
	FILE	*h4;
	char	sfx[4]="gp";
	char	sfxps[4]="ps";
	char	station[4];
//	char	channel[4];
	char	title[80];
	char	legend[40];
	char	postscriptfile[13];
	short	moday;

//...name output Gnuplot file
	names2(plotfile,Station,channel,sfx);
	plotfile[12]='\0';
	names2(postscriptfile,Station,channel,sfxps);
	postscriptfile[12]='\0';

//...set up start time and date for record first sample
//      moday = mmdd  ex. 1220 = December 20
  	moday = doy2moday((int) day, (int) year);
	int month = moday/100;
	int Day = moday-month*100;
	
//...set up title
	sprintf(title,"Station: %s    Channel: %s (%2.2d:%2.2d:%2.2d) %2.2d/%2.2d/%2.2d",
	Station,channel,hour,min,sec,month,Day,year);

//...set up legend for channel orientation
	switch(channel[2]) {
	case 'Z' :
	sprintf(legend,"Vertical Motion");
	break;
	case 'E' :
	sprintf(legend,"East Motion");
	break;
	case 'N' :
	sprintf(legend,"North Motion");
	break;
	case '1' :
	sprintf(legend,"Horizontal 1 Motion");
	break;
	case '2' :
	sprintf(legend,"Horizontal 2 Motion");
	break;
	default :
	sprintf(legend," ");
	}  //endswitch

//...open output plotfile file
        if( (h4=fopen(plotfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",plotfile);
          abort();
        }
	fprintf(h4,"#Generated by sac2seg.c \n");
	fprintf(h4,"# Default is live plot \n");
	fprintf(h4,"# For fixed image file remove # on TYPE=image \n");
	fprintf(h4,"# and rerun:  gnuplot %s \n",plotfile);
#ifdef WIN
  	fprintf(h4,"TYPE=\"live\" \n"); 
  	fprintf(h4,"#TYPE=\"image\" \n"); 
  	fprintf(h4,"if (TYPE ne \"live\") { \n"); 
  	fprintf(h4,"set terminal jpeg large size 800,600 enhanced \n"); 
  	fprintf(h4,"set output \"%s\" \n",postscriptfile); 
#else
  	fprintf(h4,"TYPE=\"live\" \n"); 
  	fprintf(h4,"#TYPE=\"image\" \n"); 
  	fprintf(h4,"if (TYPE ne \"live\") { \n"); 
  	fprintf(h4,"set terminal postscript size 8.0,6.0 in color \\\n"); 
	fprintf(h4,"portrait enhanced font \'Helvetica,10\' \n");
  	fprintf(h4,"set output \"%s\" \n",postscriptfile); 
#endif
  	fprintf(h4," } else { \n"); 
	fprintf(h4,"TRM=GPVAL_TERM                   \n");
	fprintf(h4,"if (TRM eq \"unknown\" ) {set terminal x11 persist} \n");
	fprintf(h4,"if (TRM eq \"qt\") { set terminal wxt size 800,600 persist  } \n");
	fprintf(h4,"if (TRM eq \"wxt\") { set terminal wxt size 800,600 persist }   \n");
	fprintf(h4,"if (TRM eq \"x11\") { set terminal x11 persist } \n");
  	fprintf(h4," }  \n"); 
	fprintf(h4,"                                 \n");
	fprintf(h4,"set grid \n");
	fprintf(h4,"set title \"%s\" \n",title); 
	fprintf(h4,"set xlabel \"Time (sec)\" \n");
	fprintf(h4,"set ylabel \"Amplitude\" \n");
	fprintf(h4,"p '%s' u 1:2 w l lc rgb \"blue\" t \"%s\" \n",datfile,legend);
//	fprintf(h4,"set terminal postscript portrait color size 7,4 \n");
//	fprintf(h4,"set output \"%s\" \n",postscriptfile);
//	fprintf(h4,"replot \n");
	fclose(h4);
	} //end gengnuplot
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: findnptshort                                |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: find a short value to break up trace into sub     |
   |          traces.                                           |
   |------------------------------------------------------------|
*/
	int  findnptshort(int npts)
	{
	int	nmax = 65535;
	int	j=1;
	int	j2;
	int	nptshort;
	int	remain,
		itest;
	int	jsave;
	nptshort = npts;
	while (nptshort > nmax) {
	j = j + 1;
	nptshort = npts/j;
		if (j>1000) {
		return(0);
		}
	} //endwhile

//...look for a smaller residual
	jsave = j;
	itest = npts%nptshort;
	for (j2=1; j2<10; j2++) {
	remain = npts%(npts/(j+j2));
		if(remain < itest ) {
		itest=remain;
		jsave = j+j2;
		} //endif
	} //next j2	
	nptshort = npts / jsave;
	return(nptshort);
	} //end findnptshort
