! |$Id: bzrt.F90,v 1.1.1.1 2024/10/07 17:24:38 pm Exp $          |
! --------------------------------------------------------------
! | Basic Seismic Utilities (BSU)                              |
! | LANGUAGE: Fortran77                                        |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: bzrt.f                                       |
! | DIRECTORY: bsu-3.0.3/src/Fort/bsegy                        |
! | REVISION: Double precision version PLPLOT                  |
! |------------------------------------------------------------|
! | Determine tool orientation from PCA of hodogram            |
! | Note on Sign Convention for angles:                        |
! | Azimuth + clockwise from north=0deg Az                     |
! | Tan function Fortran, measured from east, + counter clockwz|
! |------------------------------------------------------------|
! |  NOTICE:                                                   |
! |  Copyright (C) 2021 Paul Michaels                          |
! |  <paulmichaels@boisestate.edu>                             |
! |  This program is free software; you can                    |
! |  redistribute it and/or modify it under the terms          |
! |  of the GNU General Public License as published            |
! |  by the Free Software Foundation; either version           |
! |  3 of the License, or (at your option) any later           |
! |  version.  This program is distributed in the              |
! |  hope that it will be useful, but                          |
! |  WITHOUT ANY WARRANTY; without even the implied            |
! |  warranty of MERCHANTABILITY or FITNESS FOR A              |
! |  PARTICULAR PURPOSE.  See the GNU General Public           |
! |  License for more details.                                 |
! |  You should have received a copy of the GNU                |
! |  General Public License along with this program;           |
! |  if not, write to the Free Software Foundation,            |
! |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
! |------------------------------------------------------------|
! | LIBRARY REQUIRED:  PLPLOT                                  |
! |                                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |                                                            |
! |REQUIRES: LAPACK (for eigenvalue decomposition)             |
! |http://www.netlib.org/lapack/                               |
! |------------------------------------------------------------|
! |  PARAMETERS:                                               |
! |  ndim = maximum number of samples in a seismic trace       |
! |------------------------------------------------------------|
! |  INCLUDE FILE REQUIRED:                                    |
! |  ../include/bsegy.f90    (modified segy header definitions)|
! |                (no reel header, floating point data)       |
! |------------------------------------------------------------|
! |  SUBROUTINES CALLED:                                       |
! | In-line===============                                     |
! |     GETPRM  get input parameters from command line or      |
! |             interactive query.  Should be inline with      |
! |             main program, specific to application.         |
! |     LSTPRM  echo check of input parameters from GETPRM.    |
! |             Values written to a history file (*.lst)       |
! |     HODPLT  Plots hodograms for QC. Requires PLPLOT package|
! |     FNDANG  Converts PCA vectors to azimuth angles based   |
! |             on a control target vector which resolves      |
! |             180 degree ambiguity (need to observe tool exit|
! |             from borehole in the field)                    |
! |     CORMAT  computes covariance matrix                     |
! |     OUTNIT  prints a matrix to listing file                |
! |     EIGEN   computes eigenvalues and eigenvectors          |
! |             Uses LAPACK: call ssyev()                      |
! |     SORT    sorts eigenvalues, drags eigenvectors along    |
! |             Sort is necessary, since major motion          |
! |             corresponds to larges eigenvalue, hence that   |
! |             eigenvector corresponds to R-component of my   |
! |             down-hole phone (may be different element on   |
! |             your phone, depends on wiring)                 |
! |                                                            |
! | Located in bsu-3.0.3/src/sublibF4===========               |
! |     CHKTRC  get number of samples, sample interval, and    |
! |             number of traces in the input data file.       |
! |             Also returns values for progress bar, stdout   |
! |     BSEGIN  Reads a seismic trace.                         |
! |     BSEGOUT Write a seismic trace.                         |
! |     PLTBAR  Writes progress bar on stdout                  |
! |                                                            |
! --------------------------------------------------------------
!
!
!
      include '../include/bsegy.f90'
      parameter (ndim=65536)
      parameter (ia=2)
!
      real*4            s1(ndim),s2(ndim)
      real*8            ampsum
      character*4       aplc
      character*40      infil, outfil, outlst, hodlst
      character*40      pltfil
      byte              hdr(240),hdt(240)
      character*80      card
      integer*1         bz
      character*1024    cmd
      integer*1         hsw1
!
      common /header/ hd
!
!////////////////\\\\\\ svd \\\\\\\\\\\/////////////////////////
!     integer p
      real*4    a(ia,ia),eigen(ia)
!///////////////////\\\\\\\\ pca \\\\\\\///////////////////////
      real*4 w(ia*2)
      real*4 xsia(ndim*2)
      real*4 xsibar(ndim*2)
!
!...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
!
!...define application prefix
      write(aplc,'(''bzrt'')')
!
!...get parameters-------------------------------------------
      call GETPRM(nargsx,infil, &
      jchv,jchh,ipct,hsw1,tmina,tmaxa)
!
!...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil, &
      bar,invbar,ibar,ntrace,ndim)
!
!...compute start and end samples for desired window
      jstart = tmina/fsamin + 1
      if (jstart.lt.1) then
      write(*,'("ABORT: tmin less than 0")')
      stop
      endif
      jend = tmaxa/fsamin + 1
      if (jend.gt.npts) then
      write(*,'("ABORT: tmax too large for number of samples")')
      stop
      endif
      if (jend.le.jstart) then
      write(*,'("ABORT: Window Error, tmin > tmax")')
      stop
      endif
!
!...define output image file
!      write(pltfil,'(a,a,''.ps'')') aplc,infil(1:4)
      write(pltfil,'(''bzrt.ps'')') 
!
!...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,pltfil, &
      infil,jchv,jchh,ipct,hsw1,tmina,tmaxa,jstart,jend)
!
!
!...open output listing file...
      io09=9
      write(outfil,'(a,''pca.dat'')') aplc
      open(io09,file=outfil,access='sequential',position="append")
!
      icall1=0
      icall2=0
!
!...data loop=======================================....
!
!...read R-component
      ntr = 0
      jrec=jchv
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,jrec,iexit)
!...save header
      do  j=1,240
      hdr(j)=hd(j)
      enddo   
!
       ntr = ntr + 1
!...display progress....
      call pltbar(bar,invbar,ibar,ntr,4)

!
!...read H-component
      jrec=jchh
      CALL BSEGIN(iunit1,ndim, &
      icall1,npts,s2,fsamin,infil,jrec,iexit)
!...save header
      do j=1,240
      hdt(j)=hd(j)
      enddo   
       ntr = ntr + 1
!...display progress....
      call pltbar(bar,invbar,ibar,ntr,4)
!
!...construct motion 2-component vectors, xsia
      amaxm=-999999.
      ampsum=0.0
      do 50 jt=jstart,jend
      amp=(s1(jt)**2+s2(jt)**2)
      ampsum=ampsum + amp
      if(amp.ne.0.0) then
      amp=sqrt(amp)
      endif
      if(amp.gt.amaxm) then
      amaxm=amp
      jmaxm=jt
      endif
  50  continue
!
!...compute average amplitude in window jstart to jend
      ampsum=ampsum/real(jend-jstart+1,8)
!
!...compute passing threshold
      thresh=(float(ipct)*amaxm)/100.
!
!
!...resample the trace for vectors above thresh
      kt=0
      do 51 jt=jstart,jend
      amp=(s1(jt)**2+s2(jt)**2)
      if(amp.ne.0.0) then
      amp=sqrt(amp)
      endif
      if(amp.ge.thresh) then
      kt=kt+1
      index1=(kt-1)*2
      xsia(index1+1)=s1(jt)
      xsia(index1+2)=s2(jt)
      endif
  51  continue
!
!...compute principal components
      call svdpca(w,xsia,kt,2,2,2,io11, &
      a,eigen,xsibar)
!
!...compute saz = angle in vertical plane
      azr=0
      azt=azr+90.
      sazR=azr+atan2(w(2),w(1))*(180/3.1415926)
      sazT=azt+atan2(w(4),w(3))*(180/3.1415926)
!
!...make sure azimuth angles positive 0<=az<=360
!      sazR=sazR+360.
!      if(sazR.gt.360.) sazR=sazR-360.
!      if(sazT.gt.360.) sazT=sazT-360.
!      if(sazT.lt.0.) sazT=sazT+360.
!
      write(io11,FMT='('' '')')
      write(io11,FMT='(50(1h-))')
      write(io11,FMT='("PCA SOLUTION:")')
      write(io11,FMT='(/"Angles With Respect to V-component",/ &
      "Estimated motion vertical plane:")')
      write(io11,FMT='("sazV=",F10.2," degrees")') sazR
      write(io11,FMT='("sazH=",F10.2," degrees")') sazT
      write(io09,FMT='(3(f10.4,1x),e14.7)') (tmina+tmaxa)/2.0, sazR, sazT,ampsum
!
!...compute azimuth for each component
      write(io11,FMT='('' '')')
      write(io11,FMT='('' '')')
      write(io11,FMT='(50(1h-))')
      write(io11,400) ipct,thresh,kt
  400 Format(2x,"Threshold=",i3,"%"/&
      2x,"Threshold Amplitude=",E14.7/&
      2x,"Number of Cases Above Threshold=",i5)
      tjmaxm=fsamin*float(jmaxm)
      write(io11,405) jmaxm,tjmaxm,amaxm
  405 Format(2x,"Maximum Amplitude at sample=",i5/ &
      2x,"Corresponding to sample time=",f10.3,"sec"/ &
      2x,"Maximum Amplitude=",E14.7)
!
!...display progress....
      ntr = ntr +1
      call pltbar(bar,invbar,ibar,ntr,4)
!
!
!...END LOOP========================================....
!
!...make postscript plot
      k=0
      do 101, i=1,kt
      k=k+1
      index1=(i-1)*2
      S1(k)=xsia(index1+1)
      S2(k)=xsia(index1+2)
 101  continue
#ifdef PLPLOT
      call  hodplt(kt,S1,S2,sazR,sazT,ofrn,jchv,jchh,ndim,pltfil,hsw1)
#else
      call  ghodplt(kt,S1,S2,sazR,sazT,ofrn,jchv,jchh,pltfil,hsw1)
      bz=0
      write(cmd,'(''gnuplot bzrt.gp'',a)') bz
      call runcmd(cmd)

#endif
!...display progress....
      ntr = ntr + 1
      call pltbar(bar,invbar,ibar,ntr,4)
!
!
!...get time and date....
      write(*,'('' '')')
      write(6,240) ntrace,outlst
240   format(/' Traces Processed = ',i5/ &
      ' LISTING FILE: ',a)
      write(*,*) "PCAFILE:  bzrtpca.lst"
      write(*,*) "PCA: Time, (Angles +/-180 WRT to V component)"
      write(6,'(" IMAGE FILE: ",a13)') pltfil
      write(io11,'(/" IMAGE FILE: ",a13)') pltfil
#ifndef PLPLOT
      write(6,*) "GNUPLOT FILE: bzrt.gp  "
      write(io11,*) "GNUPLOT FILE: bzrt.gp  "
#endif
!
!
      close(io09)
      close(io11)
      close(iunit1)
      close(iunit2)
!     stop
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: getprm.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! | get input parameters or help                               |
! --------------------------------------------------------------
!

      subroutine GETPRM(nargsx,infil, &
      jchv,jchh,ipct,hsw1,tmina,tmaxa)
!
      character*40      infil
      character*40      arg1,arg2,arg3,arg4
      character*40      arg5,arg6,arg7
      integer*1         hsw1
!
!
!...get input/output file names...
!   |--------------------------------------------|
!   | file naming convention                     |
!   | application prefix 4 characters "bxxx"     |
!   | data set suffix 4 characters "dddd"        |
!   | file name = bxxxdddd.seg                   |
!   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2021  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
!
!cc---LINUX VERSION
      nargsx=iargc()
      karg=-1
!...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bzrt: Hodogram analysis by Principal     |'
      write(*,*) '| Components (PCA) (plot motion virtical)  |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*) &
      'bzrt   infile chV chH ipct hsw1 tmin tmax'
      write(*,*) ' '
      write(*,*) 'infile =input file name '
      write(*,*) 'chV    = channel with V-component (int) '
      write(*,*) 'chH    = channel with R or T-component (int) '
      write(*,*) 'ipct   = percent of max amplitude to include (int) '
      write(*,*) 'hsw1   = switch to set horizontal component ID   '
      write(*,*) '       2 = Horizontal is R-component              '
      write(*,*) '       3 = Horizontal is T-component              '
      write(*,*) '   (hsw1 sets label on horizontal axis of plot) '
      write(*,*) ' '
      write(*,*) 'tmin   = start time (seconds) of data window '
      write(*,*) 'tmax   = end time (seconds) of data window   '
      write(*,*) ' '
      write(*,*) ' '
      write(*,*) ' '
      write(*,*) ' Computed angles relative to V-component '
      write(*,*) ' 180 Degree uncertainty, angles positive clockwise'
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
!
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  enter: chV (int) Channel with V-component'
      read(*,*) jchv
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') jchv
      endif
!
      if(nargsx.lt.(4+karg)) then
      write(*,*)'  enter: chH (int) Channel with horizontal (R or T)'
      read(*,*) jchh
      else
      call getarg(3,arg3)
      read(arg3,'(i5)') jchh
      endif
!
      if(nargsx.lt.(5+karg)) then
      write(*,*)'  enter: ipct (percent of max amp to include, int)'
      read(*,*) ipct
      else
      call getarg(4,arg4)
      read(arg4,'(i5)') ipct
      endif
!
      if (nargsx.lt.(6+karg)) then
      write(*,*)'  enter: Hsw1 Horizontal Switch, H is R or T'
      write(*,*)'  hsw1  =  2  then horizontal is R component'
      write(*,*)'  hsw1  =  3  then horizontal is T component'
      read(*,*) hsw1
      else
      call getarg(5,arg5)
      read(arg5,'(i5)') hsw1
      endif
!
      if (nargsx.lt.(7+karg)) then
      write(*,*)'  enter: tmin (start time seconds)'
      read(*,*) tmina
      else
      call getarg(6,arg6)
      read(arg6,'(f10.4)') tmina
      endif
!
      if (nargsx.lt.(8+karg)) then
      write(*,*)'  enter: tmax (end time seconds)'
      read(*,*) tmaxa
      else
      call getarg(7,arg7)
      read(arg7,'(f10.4)') tmaxa
      endif
!
      return
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: lstprm.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! | echo check of input parameters, writes to a history file   |
! --------------------------------------------------------------
!
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil, &
      infil,jchv,jchh,ipct,hsw1,tmin,tmax,jstart,jend)
      character*4       aplc
      character*40      outlst,outfil,infil
      integer*1         hsw1
!
!...open output listing file...
      io11=10
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2000  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|---------------------------------|'
      write(io11,'('' Program bzrt  '')')
      write(io11,'('' Input File: '',a)') infil
!      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   V-Channel= '',i5)') jchv
      write(io11,'(''   H-Channel= '',i5)') jchh
      write(io11,'(''       ipct = '',i5)') ipct
      write(io11,'(''      hsw1  = '',i5)') hsw1
      write(io11,'(''      tmin  = '',f10.4)') tmin
      write(io11,'(''      tmax  = '',f10.4)') tmax
      write(io11,'(''      jstart = '',i10)') jstart
      write(io11,'(''      jend   = '',i10)') jend  
      write(io11,'(''    '')')
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
!
      return
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: svdpca.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! |                                                            |
! | svdpca: principal component analysis by eigenvalue         |
! |         decomposition of correlation matrix                |
! |                                                            |
! |  xsi =  input vector   (m x 1)                             |
! |  f   =  output vector  (n x 1)                             |
! |                                                            |
! |  w   =  weight matrix  (n x m) (each row is an eigenvector |
! |                            of the input correlation matrix)|
! |  ndim = dimension of output vector = n                     |
! |  mdim = dimension of input vector  = m                     |
! |                                                            |
! |  <<<<<<<<<<<<<<< version for use with bzrt.f >>>>>>>>>>>>  |
! |------------------------------------------------------------|
! |  subroutines used:                                         |
! |   eigen.f  sort.f cormat.f outnit.f                        |
! |------------------------------------------------------------|
!
      subroutine svdpca(w,xsia,nvec,mdim,ndim,ia,io11, &
      a,eigenv,xsibar)
      implicit real*4 (a-h,o-z)
!////////////////\\\\\\ svd \\\\\\\\\\\/////////////////////////
      real*4    a(ia,ia),eigenv(ia)
!///////////////////\\\\\\\\ pca \\\\\\\///////////////////////
      real*4 w(*)
      real*4 xsia(*)
      real*4 xsibar(*)
!
!
!  <<<<<<<<<<<<<<< begin input block>>>>>>>>>>>>>>>>>>
!...read data
      if(ndim.gt.ia) then
      write(*,*) 'abort--ndim too large'
      stop
      endif
      if(mdim.gt.ia) then
      write(*,*) 'abort--mdim too large'
      stop
      endif
!
      do 5 k=1,mdim
      xsibar(k)=0.0
    5 continue
!
      do 10 j=1,nvec
      do 12 k=1,mdim
      index1=(j-1)*mdim+k
!...compute average input vector
      xsibar(k)=xsibar(k)+xsia(index1)/float(nvec)
   12 continue
   10 continue
!
!  <<<<<<<<<<<<<<<end of input block>>>>>>>>>>>>>>>>>>>
!
      write(io11,'(''  average input vector:''/)')
      write(io11,'(5(1x,e14.7))') (xsibar(k),k=1,mdim)
!
!...remove mean from input vectors
      do 120 j=1,nvec
      do 100 k=1,mdim
      index1=(j-1)*mdim+k
      xsia(index1)=xsia(index1)-xsibar(k)
  100 continue
  120 continue
!
!...compute <correlation matrix>
      call cormat(nvec,mdim,xsia,a,ia)
!
!...set dimensions
      n=mdim
!
!...perform eigenvalue decomposition
      write(io11,'(//''  Correlation Matrix'')')
      call outnit(a,n,n,ia)
!
      call eigen(a,n,eigenv,ia)
!
      write(io11,'(//''  Eigenvalues'')')
      call outnit(eigenv,n,1,ia)
!
      write(io11,'(//''  Eigenvectors '')')
      call outnit(a,n,n,ia)
!
!...copy eigenvectors into weight matrix, w
      do 75 jx=1,ndim
      indxj=(jx-1)*mdim
      do  kx=1,mdim
      w(indxj+kx)=a(kx,jx)
      enddo
   75 continue
!
!...final  ////////////////////////////
!...restore mean to vectors
      do 130 j=1,nvec
      do  k=1,mdim
      index1=(j-1)*mdim+k
      xsia(index1)=xsia(index1)+xsibar(k)
      enddo
  130 continue
!
!
      write(io11,'(//''  weights: Row Vectors (R,T)''/)')
!     write(*,'(//''  weights:''/)')
      do 62 i=1,ndim
      index1=(i-1)*mdim
      write(io11,901) (w(index1+kk),kk=1,mdim)
!     write(*,901) (w(index1+kk),kk=1,mdim)
  901 format(5(2x,e10.4))
   62 continue
!
      return
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: cormat.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! |                                                            |
! | cormat:  computes correlation matrix summed over           |
! |          number of exemplars                               |
! |------------------------------------------------------------|
      subroutine cormat(nvec,mdim,xsia,matrix,ia)
      implicit real*4 (a-h,o-z)
      real*4 xsia(*),matrix(ia,ia)
!
!...zero correlation matrix
      do 10 j=1,mdim
      do  k=1,mdim
      matrix(j,k)=0.00
      enddo
   10 continue
!
!...loop over number of vectors
      do 100 jv=1,nvec
!
      index1=(jv-1)*mdim
      do 50 j=1,mdim
      indxj=index1+j
      do  k=1,mdim
      indxk=index1+k
      matrix(j,k)=matrix(j,k) + xsia(indxj)*xsia(indxk)
      enddo
   50 continue
!
  100 continue
      return
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: eigen.f                                      |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! | eigen.f      find eigenvalues and eigenvectors for symetric|
! |              matrix using LAPACK routines                  |
! |------------------------------------------------------------|
! |  inputs:                                                   |
! |  a= matrix (n x n) input symetric matrix                   |
! |  ia=physical dimension                                     |
! |  n=logical dimension                                       |
! |                                                            |
! |------------------------------------------------------------|
! |  outputs:                                                  |
! |           w=eigenvalues                                    |
! |           a=matrix of eigenvectors (columns)               |
! |------------------------------------------------------------|
      subroutine eigen(a,n,w,ia)
      parameter (ldim=1000)
      dimension a(ia,*),w(*)
      character*1  jobz,uplo
      real work(ldim)
      integer info
!
!...define some parameters
!     jobz=v compute eigenvalues and eigenvectors
!     jobz=N compute eigenvalues only
      jobz='V'
      uplo='U'
      lda=n
      lwork=3*n-1
      if(lwork.gt.LDIM) then
      write(*,*) 'ABORT-work dimension exceeded in eigen.f'
      stop
      endif
!
      call ssyev(jobz,uplo,n,a,lda,w,work,lwork,info)
      call sort(w,a,n,ia)
!
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: sort.f                                       |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! |                                                            |
! | sort: sorts eigenvalues and eigenvectors simultaneously    |
! |       a(1)=maximum  a(n)=min                               |
! |------------------------------------------------------------|
! |  inputs:                                                   |
! |  a= vector of eigenvalues                                  |
! |  u= matrix of eigenvectors                                 |
! |                                                            |
! |------------------------------------------------------------|
! |  outputs:                                                  |
! |  a= sorted                                                 |
! |  u= sorted                                                 |
! |                                                            |
! |------------------------------------------------------------|
      subroutine sort(a,u,n,ia)
      dimension a(ia),u(ia,ia)
      lim=n-1
    1 int1=n
      do 5 i=1,lim
      k=n+1-i
      j=k-1
!      if (a(k)-a(j)) 5,5,6
      if(a(k).gt.a(j)) then
!...interchange if a(k).gt.a(j)
    6 temp=a(j)
      a(j)=a(k)
      a(k)=temp
      int1=k
      do 8 l=1,n
      hold1=u(l,j)
      u(l,j)=u(l,k)
      u(l,k)=hold1
    8 continue
      endif
    5 continue
      if(int1.eq.n) return
      go to 1
      end
!
! |------------------------------------------------------------|
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: outnit.f                                     |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! |------------------------------------------------------------|
! |                                                            |
! | output: outputs matrix                                     |
! |                                                            |
! |------------------------------------------------------------|
! |  inputs:                                                   |
! |  a= matrix to list                                         |
! |  dimensions a(n,m) logical                                 |
! |             a(ia,ia) physical                              |
! |                                                            |
! |------------------------------------------------------------|
! |  outputs:                                                  |
! |  list of matrix to file, unit 10                           |
! |                                                            |
! |                                                            |
! |------------------------------------------------------------|
      subroutine outnit(a,n,m,ia)
      real a(ia,ia)
      kkt=m/5
      kt=5
      kb=1
      if (m.lt.5) go to 71
      do 70 kk=1,kkt
      do 60 j=1,n
!     write(*,218) j,(a(j,k),k=kb,kt)
      write(10,218) j,(a(j,k),k=kb,kt)
   60 continue
      write(10,209)
      write(10,209)
!     write(*,209)
!     write(*,209)
      kb=kt+1
      kt=kt+ 5
   70 continue
   71 continue
      do 80 j=1,n
!     write(*,218) j,(a(j,k),k=kb,m)
      write(10,218) j,(a(j,k),k=kb,m)
   80 continue
  209 format(1x)
  218 format(/1x,i3,5(2x,e11.5))
      return
      end
#ifdef PLPLOT
!#
! --------------------------------------------------------------
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | subroutine: hodplt.f                                       |
! |                                                            |
! | copyright (c) 2002 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! | Plots a hodogram as points                                 |
! |                                                            |
! | Note on Sign Convention for angles:                        |
! | Azimuth + clockwise from north=0deg Az                     |
! | Tan function Fortran, measured from east, + counter clockwz|
! |REQUIRES PLPLOT:                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |            r is vertical, t is horizontal                  |
! --------------------------------------------------------------
      subroutine hodplt(npts,r,t,azr,azt,ofrn,jchv,jchh,ndim,pltfil,hsw1)
      use plplot
      real*4 r(npts),t(npts)
      real*8 xd(npts),yd(npts)
      real*8 x(2),y(2)
      real*8 fdef,fscale,fzero
      real*8 fmargin,aspect,yjust,xjust
      real*8 xmn,xmx,ymn,ymx
      real*8 amaxm
      real*8 fwidth
      integer*1 hsw1
!
      integer*4 ofrn
      character*40 pltfil
      character*60 titl
      character*40 xlbl
      character*40 ylbl
      character*7  Rstrg
      character*7  Tstrg
!
!...open device
!  Portrait
      izero=0
      ione=1
!...copy to double precision
      do 102, i=1,npts
      xd(i)=r(i)
      yd(i)=t(i)
102   continue
      call plscolor(izero)
      call plsori(ione)
!...aspect ratio needed for portrait, unless you don't mind
!   plots which are squished in the vertical dimension.
!   aspect=8.5/11.0
          fmargin=0.
          aspect=0.77
          yjust=0.5
          xjust=0.0
          call plsdev('ps')
!          write(pltfil,'(i4.4,''.ps'')') ofrn
!
      call plsdidev(fmargin,aspect,yjust,xjust)
!
      call plsfnam(pltfil)
!
!...initialize plot
      nx=1
      ny=1
      call plinit()
      call plssub(nx,ny)
      call plfontld(izero)
!
!...determine maximum and minimums
      fmax=r(1)**2+t(1)**2
      if (fmax.ne.0.0) fmax=sqrt(fmax)
      fmin=fmax
      do 10, i=2,npts
      tst=r(i)**2+t(i)**2
      if(tst.ne.0.0) tst=sqrt(tst)
      fmax=max(tst,fmax)
      fmin=min(tst,fmin)
   10 continue
      amaxm=max(abs(fmax),abs(fmin))
!
!...set environment, window and viewport
!      call plenv(-amaxm,amaxm,-amaxm,amaxm,0,0)
      izero=0
      ione=1
      iwidth=1
      fwidth=1.0
      fdef=0.
      fscale=0.5
      call pladv(izero)
      call plschr(fdef,fscale)
      call plprec(izero,izero)
      call plssym(fdef,fscale)
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      xmn=0.1
      xmx=.55
      ymn=0.6
      ymx=.95
      call plvpor(xmn,xmx,ymn,ymx)

      if (tsw1.gt.0.) then
      call plwind(-amaxm,amaxm,+amaxm,-amaxm)
      else
      call plwind(-amaxm,amaxm,-amaxm,+amaxm)
      endif
      fzero=0.
      izero=0
      call plbox('abcnst',fzero,izero,'abcnstv',fzero,izero)
!
      write(titl,'(''Angles WRT V: Shot='',i4.4,'' Angles='',f6.1, &
      '' '',f6.1)') ofrn,azr,azt
      write(xlbl,'(''V-Comp. ch='',i2.2)') jchv
      if(hsw1.eq.2) then
      write(ylbl,'(''H Comp is R ch='',i2.2)') jchh
      else
      write(ylbl,'(''H-Comp. is T ch='',i2.2)') jchh
      endif
      call pllab(ylbl,xlbl,titl)
!
      icode=9
!      call plpoin(npts,r,t,icode)
      call plpoin(yd,xd,icode)
!
!...plot lines along determined azimuths
      x(1)=0.
      y(1)=0.
      angl=3.1415926/180.0
      y(2)=(amaxm/2.)*cos((azr)*angl)
      x(2)=(amaxm/2.)*sin((azr)*angl)
      write(Rstrg,'(f6.1)') azr
      call plptex(x(2),y(2),fzero,fzero,fzero,Rstrg)
      call plline(x,y)
      y(2)=(amaxm/2.)*cos((azt)*angl)
      x(2)=(amaxm/2.)*sin((azt)*angl)
      write(Tstrg,'(f6.1)') azt
      call plptex(x(2),y(2),fzero,fzero,fzero,Tstrg)
      call plline(x,y)
      call plend()
      return
      end

#else
!#
! --------------------------------------------------------------
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | subroutine: ghodplt.f                                      |
! |                                                            |
! | copyright (c) 2017 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! | Plots a hodogram as points                                 |
! |                                                            |
! | Note on Sign Convention for angles:                        |
! | Azimuth + clockwise from north=0deg Az                     |
! | Tan function Fortran, measured from east, + counter clockwz|
! |REQUIRES GNUPLOT:                                           |
! |GNUPLOT is GNU plot library which supports C pipes and calls|
! |The program is to write out a *.gp gnuplot file and then    |
! |use runcmd() interface to execute a system call from C.     |
! |See BSU sublibC for the runcmd.c code whose object module is|
! |linked to this Fortran 90 program.                          |
! |                                                            |
! --------------------------------------------------------------
      subroutine ghodplt(npts,r,t,azr,azt,ofrn,jchv,jchh,pltfil,hsw1)
      real*4 r(npts),t(npts)
      real*4 x(2),y(2)
      real*4 Tx,Ty,Rx,Ry
      integer*4 ofrn
      character*80 sxrange,syrange
      character*40 pltfil
      character*7  Rstrg
      character*7  Tstrg
      integer*1    hsw1
! fill outfile name
! Note: Microsoft Windows does not work well with Postscript
! so compile for a jpeg output.
#ifdef WIN
      write(pltfil,'("bzrt.jpeg")')
#else
      write(pltfil,'("bzrt.ps")')
#endif
!
!...determine maximum and minimums
      fmax=r(1)**2+t(1)**2
      if (fmax.ne.0.0) fmax=sqrt(fmax)
      fmin=fmax
      do 10, i=2,npts
      tst=r(i)**2+t(i)**2
      if(tst.ne.0.0) tst=sqrt(tst)
      fmax=max(tst,fmax)
      fmin=min(tst,fmin)
   10 continue
      amaxm=max(abs(fmax),abs(fmin))
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")')-amaxm,amaxm 
      if (tsw1.gt.0) then
      write(syrange,'("set yrange [",F10.2,":",F10.2,"]")')+amaxm,-amaxm
      else
      write(syrange,'("set yrange [",F10.2,":",F10.2,"]")')-amaxm,+amaxm
      endif
! compute locations for labels
      angl=3.1415926/180.0
      Rx=(amaxm/2.)*cos((azr)*angl)
      Ry=(amaxm/2.)*sin((azr)*angl)
      Tx=(amaxm/2.)*cos((azt)*angl)
      Ty=(amaxm/2.)*sin((azt)*angl)
      open(unit=12,file='bzrt.gp',access='sequential')
      write(12,*) "# Generated by bzrt.F90  Subroutine ghodplt()     "
#ifdef WIN
      write(12,*) "set terminal jpeg large size 500,500 enhanced"
      write(12,*) "set output '\"
      write(12,100) pltfil
  100 format(a9,"'")
#else
      write(12,*) "set terminal postscript size 5.0,5.0 in \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output '\"
      write(12,100) pltfil
  100 format(a7,"'")
#endif 
      write(12,*) "set grid                                          "
      write(12,*) "set nokey" 
      write(12,*) "set ylabel '\"                                    
      write(12,'("V-Component, ch=",i2.2," \")') jchv
      write(12,*) " ' "
      write(12,*) "set xlabel '\"
      if(hsw1.eq.2) then
      write(12,'("H-Component = R ch=",i2.2," \")') jchh
      else
      write(12,'("H-Component = T, ch=",i2.2," \")') jchh
      endif
      write(12,*) " ' "
      write(12,*) "set title 'Angles WRT V: Shot=\"
      write(12,'(i4.4," Angles=",f6.1," ",f6.1,"\")') ofrn,azr,azt
      write(12,*) " '"
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(Rstrg,'(f6.1)') azr
      write(12,110) Rstrg,Ry,Rx
  110 format("set label '",a6,"' at ",E14.7," , ",E14.7," right")
      write(Tstrg,'(f6.1)') azt
      write(12,120) Tstrg,Ty,Tx
  120 format("set label '",a6,"' at ",E14.7," , ",E14.7," right")
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2) w p pt 6 ps 1, \"
      write(12,*) " '' u ($1):($2) w l ls 1  , \"
      write(12,*) " '' u ($1):($2) w l ls 1 "
      do 20, j=1,npts
   20 write(12,'(2(E14.7,1x))') t(j),r(j)
      write(12,*) "e "
!
!...plot lines along determined azimuths
      x(1)=0.
      y(1)=0.
      x(2)=Rx
      y(2)=Ry
      do 30, j=1,2
   30 write(12,'(2(E14.7,1x))') y(j),x(j)
      write(12,*) "e "
      x(2)=Tx
      y(2)=Ty
      do 40, j=1,2
   40 write(12,'(2(E14.7,1x))') y(j),x(j)
      write(12,*) "e "
      close(12)
      return
      end

#endif
