! |$Id: hvsr.F90,v 1.1.1.1 2024/10/07 17:24:38 pm Exp $          |
! --------------------------------------------------------------
! | Basic Seismic Utilities (BSU)                              |
! | LANGUAGE: Fortran77                                        |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: hvsr.F90                                     |
! | DIRECTORY: bsu-3.0.3/src/Fort/bsegy                        |
! |------------------------------------------------------------|
! | Program to compute Horizontal to Vertical Spectral Ratio   |
! | and plot the result.                                       |
! |------------------------------------------------------------|
! |  NOTICE:                                                   |
! |  Copyright (C) 2021 Paul Michaels                          |
! |  <paulmichaels@boisestate.edu>                             |
! |  This program is free software; you can                    |
! |  redistribute it and/or modify it under the terms          |
! |  of the GNU General Public License as published            |
! |  by the Free Software Foundation; either version           |
! |  3 of the License, or (at your option) any later           |
! |  version.  This program is distributed in the              |
! |  hope that it will be useful, but                          |
! |  WITHOUT ANY WARRANTY; without even the implied            |
! |  warranty of MERCHANTABILITY or FITNESS FOR A              |
! |  PARTICULAR PURPOSE.  See the GNU General Public           |
! |  License for more details.                                 |
! |  You should have received a copy of the GNU                |
! |  General Public License along with this program;           |
! |  if not, write to the Free Software Foundation,            |
! |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
! |------------------------------------------------------------|
! |  PARAMETERS:                                               |
! |  ndim = maximum number of samples in a seismic trace       |
! |------------------------------------------------------------|
! |  INCLUDE FILE REQUIRED:                                    |
! |  ../include/bsegy.inc    (modified segy header definitions)|
! |                (no reel header, floating point data)       |
! |------------------------------------------------------------|
! |  SUBROUTINES CALLED:                                       |
! | In-line===============                                     |
! |     GETPRM  get input parameters from command line or      |
! |             interactive query.  Should be inline with      |
! |             main program, specific to application.         |
! |     LSTPRM  echo check of input parameters from GETPRM.    |
! |             Values written to a history file (*.lst)       |
! | Located in bsu-3.0.3/src/sublibF4===========               |
! |     CHKTRC  get number of samples, sample interval, and    |
! |             number of traces in the input data file.       |
! |             Also returns values for progress bar, stdout   |
! |     BSEGIN  Reads a seismic trace.                         |
! |     BSEGOUT Write a seismic trace.                         |
! |     PLTBAR  Writes progress bar on stdout                  |
! |     DRUM    Modified E.A. Robinson's phase unwrapping sub  |
! |     RECT    Converts from Polar to Rectangular form        |
! |     NRAD2   Determines power of two number of samples      |
! |     CZERO   Zeros a complex signal                         |
! |     NLOGN   E.A. Robinson's radix 2 FFT                    |
! |     POLAR   E.A. Robinson's conversion;rectangular to polar|
! |                                                            |
! --------------------------------------------------------------
!
      include '../include/bsegy.f90'
      parameter (ndim=65536)
!
      character*5       aplc
      character*40      infil, outfil, outlst
      integer           chV,chH
      complex           Z,I
      real*4            PI
      common /header/ hd
!
      real*4, allocatable :: s1(:),s2(:) 
      real*4, allocatable :: phi1(:),g(:),f1(:),a1(:)
      real*4, allocatable :: phi2(:),f2(:),a2(:)
      real*4, allocatable :: ampV(:),ampH(:),ampHV(:),freq(:)
      complex, allocatable :: xV(:),xH(:)
      allocate(s1(ndim),s2(ndim))
      allocate(phi1(ndim),g(ndim),f1(ndim),a1(ndim))
      allocate(phi2(ndim),f2(ndim),a2(ndim))
      allocate( ampV(ndim),ampH(ndim),ampHV(ndim),freq(ndim))
      allocate(xV(ndim),xH(ndim))
!
!...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
!
!...set constants  
      PI=3.141592653589793
      I=cmplx(0,1)
!
!..."iopt" parameter selects type of computation, simple FFT or All Pole.
!   All Pole computes inverse of minimum phase wavelet assumed in autocorrelation
!   The DFT of the inverse is then reciprocated to produce spectral estimate
!   If mpts (length of autocorrelation in All Pole) is large (ie takes in all
!   of it with  significant amplitude, then both types of computation
!   produce similar results.
!   HOWEVER, the All Pole (Yule Walker) computation is preferred in producing
!   a spectral estimate when the order of the process is a small value.
!   One should examine the autocorrelation to make an assesment for the order
!   of the process (length of autocorrelation to use).  Default compile is to
!   set iopt = 0, all pole estimate.
      iopt = 0
!
!..."idft" option selects either FFT or evaluation of Z-transform on unit circle
!   to compute DFT.  By computing the Z-transform on the unit circle, we can increase
!   the sampling of the spectra.  idft=0, the frequency increment is 1/(N*dt) for FFT
!   (FFT is just fast computation of DFT).  With idft=1, frequency increment is set
!   as fmax/nfmax (nfmax is an arbitrary number of frequency samples desired). For
!   short autocorrelations, idft=1 gives a smoother plot by increased sampling.
      ! if idft=0, then FFT, radix 2 
!     idft = 0
      ! if idft=1, then Sampling Z-transform on unit circle 
      ! imax=1, 1Hz samples, imax=2 0.5Hz samples, imax=4 0.25Hz samples
      idft = 1
      imax = 4
!  [ debug =0 no debug output ] [ debug = 1, then output signals]
      idebug=0
!
!...define application prefix
      write(aplc,'(''hvsr'')')
!
!...get parameters-------------------------------------------
      call GETPRM(nargsx,infil, &
      chV,chH,tmina,tmaxa,mpts,idata,wht)    
!
!...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil, &
      bar,invbar,ibar,ntrace,ndim)
      fmax=1.0/(2.0*fsamin)
!
!...name outputfile
      write(outfil,'(a,a,''.dat'')') aplc(1:4),infil(1:4)

!
      icall1=0
      icall2=0
      ntr=0
!
!...data loop=======================================....
!
!...input Vertical Motion trace
      call BSEGIN(iunit1,ndim, &
      icall1,npts,s1,fsamin,infil,chV,iexit)
!
      ntr = ntr + 1
!
!...display progress....
      call pltbar(bar,invbar,ibar,ntr,ntrace)
!
!...input Horizontal Motion trace
      call BSEGIN(iunit1,ndim, &
      icall1,npts,s2,fsamin,infil,chH,iexit)
!
      ntr = ntr + 1
!
!...display progress....
      call pltbar(bar,invbar,ibar,ntr,ntrace)
!
!...idata  If idata=1, autocorrelations are input, we assume that
!          these are 2 sided autocorrelations computed by program bxcr.f
!          Zero lag is thus assumed to be at 1/2 of the signal length.
!          For example, if npts = 1000, sample 500 is at zero lag.
!          For idata=1 CODE IGNORES tmina and tmaxa parameters.
!          Further, mpts is used to select how much of the 1 sided
!          autocorrelation resulting from the code setting of tmina and tmaxa 
!
! AUTOCORRELATIONS Input Default Setting
! jmina and jmaxa alread defined
      if (idata.eq.1) then  
      tmina = real(npts/2)*fsamin 
      tmaxa = real(npts)*fsamin 
      jmina = npts/2 + 1
      jmaxa = npts + 1
      else
!
!...compute window limits from tmina and tmaxa (idata = 0) SIGNALS Input 
!   Apply tmina and tmaxa window ---> jmina, jmaxa
      jmina=int(tmina/fsamin)+1
        if(jmina.lt.1) then
        jmina = 1
        endif
      jmaxa=int(tmaxa/fsamin)+1
        if(jmaxa.gt.npts) then
        jmaxa = npts
        endif
      endif  !endif for idata test
!
!...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,jmina,jmaxa, &
      infil,chV,chH,tmina,tmaxa,mpts,iopt,idata,wht,fsamin,npts)     
!
!...DEBUG output two traces as text
      if (idebug.eq.1) then
      open(15,file="DEBUG.lst",access='sequential')
      do 97 j=jmina,jmaxa
      t=float(j-1)*fsamin
      write(15,FMT='(f10.4,2x,e14.7,2x,e14.7)') t, s1(j),s2(j)  
   97 continue
      close(15)
      endif
!
!...compute radix 2 (first power of 2 greater than npts)
      call nrad2(npts,npr2)
      npr2h=npr2/2
      if(idft.eq.0) then
      nfmax=npr2h           ! set sampling radix 2 algorithm df=1/(N*dt)
      else
      nfmax=int(imax*fmax+ 0.5)  ! set sampling on unit circle at 1/imax Hz
      endif
!
!...zero out complex vectors (padding for below)
      call czero(npr2,xV)
      call czero(npr2,xH)
!
!...idata  If idata=1, autocorrelations are input, we assume that
!          these are 2 sided autocorrelations computed by program bxcr.f
!          Zero lag is thus assumed to be at 1/2 of the signal length.
!          For example, if npts = 1000, sample 500 is at zero lag.
!
!...compute one sided autocorrelations
!   number of lags to compute is mpts
!   window (time gate) tmina to tmaxa
      if (idata.eq.0) then
      call xcor1(npts,s1,phi1,fsamin,mpts,tmina,tmaxa) ! Vertical
      call xcor1(npts,s2,phi2,fsamin,mpts,tmina,tmaxa) ! Horizontal
      else
!     copy input signals which are autocorrelations
!     from jmina= zero lag sample
      jt=jmina
      do 54, j=1,mpts
      phi1(j)=s1(jt)  ! Vertical
      phi2(j)=s2(jt)  ! Horizontal
      jt = jt + 1
   54 continue 
      endif
!
!...normalize autocorrelations
      call xmax(mpts,phi1,pmax1,pmin1)
      call xmax(mpts,phi2,pmax2,pmin2)
      do j=1,mpts
      phi1(j)=phi1(j)/pmax1
      phi2(j)=phi2(j)/pmax2
      enddo
!
!...set Whitening
      w=wht/100.0  ! wht is percent
      w = w + 1.0
!
!... Apply Whitening to avoid division by zero in ratio H/V
      phi1(1)=phi1(1)*w !apply whitening
      phi2(1)=phi2(1)*w !apply whitening
!
!... output autocorrelations
      open(15,file="hvsrauto.dat",access='sequential')
      write(15,*) "Lags       Vertical      Horizontal"
      do 99 j=1,mpts
      write(15,FMT='(i5.5,2x,e14.7,2x,e14.7)') j, phi1(j),phi2(j)  
   99 continue
      close(15)
!
!...alternative spectral estimates
      if (iopt.eq.0) then   !Yule Walker All Pole 
!
!...RHS of normal equations
      do 300, j=1,mpts
      g(j)=0.0
  300 continue
      g(1)=1.0
!
!...compute minimum phase inverses of signals
      call eureka(mpts,phi1,g,f1,a1)
!
!...RHS of normal equations
      do 305, j=1,mpts
      g(j)=0.0
 305  continue
      g(1)=1.0
!
!...compute minimum phase inverses of signals
      call eureka(mpts,phi2,g,f2,a2)
!
!...compute DFT of Vertical Inverse
      signv=-1.0
      j=1
      do 109 jt=1,mpts
      xV(j)=cmplx(a1(jt),0.0) ! inverse operator
      j=j+1
 109  continue
!
      write(*,'(3(2x,i5),f10.4)') npr2,mpts,nfmax,fmax
      if(idft.eq.0) then
      call nlogn(npr2,xV,signv)  ! FFT computes DFT
      else
!--------------------------------------------
      df=fmax/real(nfmax)
      do j=1,nfmax
      freq(j)=real(j-1)*df
      omega=freq(j)*2.0*PI
      xV(j)=cmplx(0.0,0.0)
      do k=1,mpts
      Z=cexp(-I*omega*fsamin*real(k-1))
      xV(j)=Z*cmplx(a1(k),0.0)+xV(j)
      enddo   
      enddo    
!-------------------------------------------------
      endif
!
!...compute DFT of Horizontal Inverse
      j=1
      do 110 jt=1,mpts
      xH(j)=cmplx(a2(jt),0.0) ! inverse operator
      j=j+1
 110  continue
      if(idft.eq.0) then
      call nlogn(npr2,xH,signv) ! FFT computes DFT
      else
!--------------------------------------------
      df=fmax/real(nfmax)
      do j=1,nfmax
      freq(j)=real(j-1)*df
      omega=freq(j)*2.0*PI
      xH(j)=cmplx(0.0,0.0)
      do  k=1,mpts
      Z=cexp(-I*omega*fsamin*real(k-1))
      xH(j)=Z*cmplx(a2(k),0.0)+xH(j)
      enddo
      enddo
!-------------------------------------------------
      endif
!
!...amplitudes of complex values of inverse of inverse and ratio
      if(idft.eq.0) then
      df=1.0/(float(npr2)*fsamin)
      iulimit=npr2h
      else
      df=fmax/real(nfmax)
      iulimit=nfmax
      endif
      write(*,'("iulimit=",i5)') iulimit
      do 120 j=1,iulimit
      ampV(j)=1.0/cabs(xV(j))
      ampH(j)=1.0/cabs(xH(j))
      ampHV(j)=ampH(j)/ampV(j)
      freq(j)=float(j-1)*df
 120  continue
      else   ! ============== DFT Estimate of Auto Input=idata=1=======
!
!...compute DFT of Vertical
      signv=-1.0
      xV(1)=cmplx(phi1(1)*w,0.0)
      do 209 jt=2,mpts
      k=npr2-jt+2
      xV(jt)=cmplx(phi1(jt),0.0)
      xV(k)=cmplx(phi1(jt),0.0)  !fill in negative time
 209  continue
!
!...compute DFT of Horizontal
      xH(1)=cmplx(phi2(1)*w,0.0)
      do 210 jt=2,mpts
      xH(jt)=cmplx(phi2(jt),0.0)
      k=npr2-jt+2
      xH(k)=cmplx(phi2(jt),0.0)
 210  continue
!
!...DEBUG output autocorrelations
      if (idebug.eq.3) then
      open(15,file="DEBUG.lst",access='sequential')
      do 98 j=1,npr2
      t=float(j-1)*fsamin
      write(15,FMT='(i5.5,2x,e14.7,2x,e14.7)') j, real(xV(j)),real(xH(j))  
   98 continue
      close(15)
      endif
!
!...Straight DFT of Autocorrelations of signal (NOT INVERSE)
      if(idft.eq.0) then
      call nlogn(npr2,xV,signv)
      call nlogn(npr2,xH,signv)
      else
      do j=1,mpts
      a1(j)=real(xV(j))
      a2(j)=real(xH(j))
      enddo
!--------------------------------------------
      df=fmax/real(nfmax)
      do j=1,nfmax
      freq(j)=real(j-1)*df
      omega=freq(j)*2.0*PI
      xV(j)=cmplx(0.0,0.0)
      do k=1,mpts
      Z=cexp(-I*omega*fsamin*real(k-1))
      xV(j)=Z*cmplx(a1(k),0.0)+xV(j)
      enddo   
      enddo    
!-------------------------------------------------
!--------------------------------------------
      df=fmax/real(nfmax)
      do j=1,nfmax
      freq(j)=real(j-1)*df
      omega=freq(j)*2.0*PI
      xH(j)=cmplx(0.0,0.0)
      do k=1,mpts
      Z=cexp(-I*omega*fsamin*real(k-1))
      xH(j)=Z*cmplx(a1(k),0.0)+xH(j)
      enddo   
      enddo    
!-------------------------------------------------
      endif
!
!...compute amplitudes of complex values and ratio
      if(idft.eq.0) then
      df=1.0/(float(npr2)*fsamin)
      iulimit=npr2h
      else
      df=fmax/real(nfmax)
      iulimit=nfmax
      endif
      do 220 j=1,iulimit
      ampV(j)=sqrt(cabs(xV(j))) ! sqrt for power to amplitude
      ampH(j)=sqrt(cabs(xH(j)))
      ampHV(j)=ampH(j)/ampV(j)
      freq(j)=float(j-1)*df
 220  continue
!
      endif   ! endof iopt
!
!...open output file
      open(12,file=outfil,access='sequential')
!
!...output to a text file
      write(12,*) "# Frequency    Vert          Horiz           H/V "
      do 130 j=1,iulimit
      write(12,140) freq(j),ampV(j),ampH(j),ampHV(j)
  140 format(f10.4,1x,3(e14.7,1x)) 
  130 continue
      close(12)
!
!...display progress....
      call pltbar(bar,invbar,ibar,ntrace,ntrace)
!
!
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/ &
      5x,'====> output in ',a/'====> listing in ',a)
!
!...plot autocorrelations
      call hvsrauto(mpts,phi1,phi2,chV,chH)
!
!...plot spectra
      call spplot(nfmax,freq,ampV,ampH,ampHV,chV,chH,fmax)
!
!...plot H/V Ratio
      call hvsrplot(nfmax,freq,ampV,ampH,ampHV,chV,chH,fmax)
!
!... deallocate arrays
      deallocate(s1,s2,phi1,g,f1,a1,phi2,f2,a2,ampV, &
      ampH,ampHV,freq,xV,xH)
!
      close(io11)
      close(iunit1)
!     stop
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: getprm.f                                     |
! |------------------------------------------------------------|
! | get input parameters or help                               |
! --------------------------------------------------------------
!
      subroutine GETPRM(nargsx,infil, &
      chV,chH,tmina,tmaxa,mpts,idata,wht)      
!
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8
      integer           chV, chH
!
!
!...get input/output file names...
!   |--------------------------------------------|
!   | file naming convention                     |
!   | application prefix 4 characters "bxxx"     |
!   | data set suffix 4 characters "dddd"        |
!   | file name = bxxxdddd.seg                   |
!   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2021  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
!
!cc---LINUX VERSION
      nargsx=iargc()
      karg=-1
!...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| hvsr: Horizontal Vertical Spectral Ratio |'
      write(*,*) '|                                          |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*) &
      'hvsr   infile chV chH tmin tmax mpts idata wht    '
      write(*,*) ' '
      write(*,*) 'infile  =input file name '
      write(*,*) 'chV     = channel with vertical '
      write(*,*) 'chH     = channel with horizontal'
      write(*,*) 'tmin    = start of time window'
      write(*,*) 'tmax    = end time window'
      write(*,*) 'mpts    = length (lags) of autocorrelation (1-sided) '
      write(*,*) 'idata   = selects type of input data'
      write(*,*) '        = 0 Time series of seismic data'
      write(*,*) '        = 1 Autocorrelations'
      write(*,*) 'wht     = whitening to bias against notches '
      write(*,*) '        = percent of zero lag auto to add to zero lag auto'
      write(*,*) ' '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
!
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  enter: chV '
      read(*,*) chV 
      else
      call getarg(2,arg2)
      read(arg2,'(I5)') chV 
      endif
!
      if(nargsx.lt.(4+karg)) then
      write(*,*) '  enter: chH '
      read(*,*) chH 
      else
      call getarg(3,arg3)
      read(arg3,'(I5)') chH 
      endif
!
      if(nargsx.lt.(5+karg)) then
      write(*,*) '  enter: tmin'
      read(*,*) tmina
      else
      call getarg(4,arg4)
      read(arg4,'(F10.0)') tmina
      endif
!
      if(nargsx.lt.(6+karg)) then
      write(*,*) '  enter: tmax'
      read(*,*) tmaxa
      else
      call getarg(5,arg5)
      read(arg5,'(F10.0)') tmaxa
      endif
!
      if(nargsx.lt.(7+karg)) then
      write(*,*) '  enter: mpts'
      read(*,*) mpts
      else
      call getarg(6,arg6)
      read(arg6,'(I10)') mpts
      endif
!
      if(nargsx.lt.(8+karg)) then
      write(*,*) '  enter: idata'
      read(*,*) idata
      else
      call getarg(7,arg7)
      read(arg7,'(I10)') idata
      endif
!
      if(nargsx.lt.(9+karg)) then
      write(*,*) '  enter: wht '
      read(*,*) wht 
      else
      call getarg(8,arg8)
      read(arg8,'(F10.0)') wht 
      endif
!
      return
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | PROGRAM NAME: lstprm.f                                     |
! |------------------------------------------------------------|
! | echo check of input parameters, writes to a history file   |
! --------------------------------------------------------------
!
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,jmina,jmaxa, &
      infil,chV,chH,tmina,tmaxa,mpts,iopt,idata,wht,fsamin,npts)       
      character*4       aplc
      character*40      outlst,outfil,infil
      integer           chV,chH
!
!...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2021  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
!
      write(io11,'('' Program master'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
!
!      write(*,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   chV = '',I5   )') chV 
      write(io11,'(''   chH = '',I5   )') chH 
!
!...check input for invalid values
      tlimit = float(npts)*fsamin
      if((tmina.lt.0).or.(tmina.gt.tlimit)) then
      write(io11,*) "tmin reset, invalid value"
      tmina = 0.
      endif
      if((tmaxa.gt.tlimit).or.(tmaxa.lt.tmina)) then
      write(io11,*) "tmax reset, invalid value"
      tmaxa = tlimit
      endif
!
      if (idata.eq.1) then
      write(io11,*) "Autocorrelation Data, tmin and tmax reset"
      endif 
      write(io11,'(''  tmin = '',f10.2)') tmina
      write(io11,'(''  tmax = '',f10.2)') tmaxa
!
!...check mpts request
      if(mpts.gt.(jmaxa-jmina+1)) then
      mpts = jmaxa-jmina + 1
      write(io11,*) "Warning: mpts reset to not exceed available autocor"
      endif
      write(io11,'(''  mpts = '',I5   )') mpts
      if (iopt.eq.0) then
      write(io11,*) "Yule Walker All Pole computation of spectrums "
      write(io11,'(''  iopt = '',I5   )') iopt
      else
      write(io11,*) "Simple DFT computation of spectrums "
      write(io11,'(''  iopt = '',I5   )') iopt
      endif
      if (idata.eq.0) then
      write(io11,*) " Input signals are seismic data "
      write(io11,'(''  idata = '', I5 )') idata
      else
      write(io11,*) " Input signals are autocorrelations "
      write(io11,'(''  idata = '', I5 )') idata
      endif
      write(io11,'(''  Whitening if any: wht ='',f4.2,'' Percent of zero lag'')') wht
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
!
      return
      end
!#
! --------------------------------------------------------------
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | subroutine: spplot.f                                       |
! |                                                            |
! | copyright (c) 2021 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! | Plots spectra for Horizontal and Vertical Components       |
! |                                                            |
! |REQUIRES GNUPLOT:                                           |
! |GNUPLOT is GNU plot library which supports C pipes and calls|
! |The program is to write out a *.gp gnuplot file and then    |
! |use runcmd() interface to execute a system call from C.     |
! |See BSU sublibC for the runcmd.c code whose object module is|
! |linked to this Fortran 90 program.                          |
! |                                                            |
! --------------------------------------------------------------
      subroutine spplot(npr2h,freq,ampV,ampH,ampHV,chV,chH,fmax)
      real*4 freq(*),ampV(*),ampH(*),ampHV(*)
      character*80 sxrange,syrange
      character*20 outfile
      character*80 cmd
      integer chV,chH
! fill outfile name
! Note: Microsoft Windows does not work well with Postscript
! so compile for a jpeg output.
#ifdef WIN
      write(outfile,'("hvsrspec.jpeg")') 
#else
      write(outfile,'("hvsrspec.ps")') 
#endif
      xmin=0.0
      xmax=fmax
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
      open(unit=12,file='hvsrspec.gp',access='sequential')
      write(12,*) "# Generated by hvsr.F90  Subroutine spplot()     "
      write(12,*) "# Default is live plot "
      write(12,*) "# for fixed image file remove # on TYPE=image"
      write(12,*) "# and rerun:  gnuplot hvsrauto.gp "
#ifdef WIN
      write(12,*) "TYPE=""live""                                    "
      write(12,*) "#TYPE=""image""                                  "
      write(12,*) "if (TYPE ne ""live"" ) {                         "
      write(12,*) "set terminal jpeg large size 500,500 enhanced"
      write(12,*) "set output '\"
      write(12,100) outfile
  100 format(a13,"'")
      write(12,*) " } else {                                      "
      write(12,*)"TRM=GPVAL_TERM                   "
      write(12,*)"if (TRM eq ""unknown"" ) {set terminal x11 0 persist} "
      write(12,*)"if (TRM eq ""qt"") { set terminal wxt 0 persist  } "
      write(12,*)"if (TRM eq ""wxt"") { set terminal wxt 0 persist } "
      write(12,*)"if (TRM eq ""x11"") { set terminal x11 0 persist } "
      write(12,*)" }"
#else
      write(12,*) "TYPE=""live""                                    "
      write(12,*) "#TYPE=""image""                                  "
      write(12,*) "if (TYPE ne ""live"" ) {                         "
      write(12,*) "set terminal postscript size 6.0,3.0 in color \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output '\"
      write(12,100) outfile
  100 format(a11,"'")
      write(12,*) " } else {                                      "
      write(12,*)"TRM=GPVAL_TERM                   "
      write(12,*)"if (TRM eq ""unknown"" ) {set terminal x11 0 persist} "
      write(12,*)"if (TRM eq ""qt"") { set terminal wxt 0 persist  } "
      write(12,*)"if (TRM eq ""wxt"") { set terminal wxt 0 persist } "
      write(12,*)"if (TRM eq ""x11"") { set terminal x11 0 persist } "
      write(12,*)" }"
#endif 
      write(12,*) "set grid                                          "
!      write(12,*) "set nokey" 
      write(12,*) "set xlabel '\"                                    
      write(12,'("Frequency (Hz) \")')
      write(12,*) " ' "
      write(12,*) "set ylabel '\"
      write(12,'("Amplitude \")') 
      write(12,*) " ' "
      write(12,*) "set title 'Horizontal and Vertical Spectra \"
      write(12,'(" H=ch",I2,"   V=ch",I2,"\")') chH,chV
      write(12,*) " '"
      write(12,'(a)') sxrange 
!      write(12,'(a)') syrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2) w l ls 1 lc rgb ""red"" t ""Horiz"", \"
      write(12,*) " '' u ($1):($2) w l ls 1 lc rgb ""blue"" t ""Vert""  , "
!      write(12,*) " '' u ($1):($2) w l ls 1 "
      do j=1,npr2h
      write(12,'(2(E14.7,1x))') freq(j),ampH(j)
      enddo
      write(12,*) "e "
      do  j=1,npr2h
      write(12,'(2(E14.7,1x))') freq(j),ampV(j)
      enddo
      write(12,*) "e "
      close(12)
      bz=0         !strings terminate with a zero in C-language
      write(cmd,'(''gnuplot hvsrspec.gp'',a)') bz
      call runcmd(cmd)
      return
      end 
!#
! --------------------------------------------------------------
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | subroutine: hvsrplot.f                                     |
! |                                                            |
! | copyright (c) 2021 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! | Plots Horizontal to vertical spectral ratio H/V(freq)      |
! |                                                            |
! |REQUIRES GNUPLOT:                                           |
! |GNUPLOT is GNU plot library which supports C pipes and calls|
! |The program is to write out a *.gp gnuplot file and then    |
! |use runcmd() interface to execute a system call from C.     |
! |See BSU sublibC for the runcmd.c code whose object module is|
! |linked to this Fortran 90 program.                          |
! |                                                            |
! --------------------------------------------------------------
      subroutine hvsrplot(npr2h,freq,ampV,ampH,ampHV,chV,chH,fmax)
      real*4 freq(*),ampV(*),ampH(*),ampHV(*)
      character*80 sxrange,syrange
      character*20 outfile
      character*80 cmd
      integer chV,chH
! fill outfile name
! Note: Microsoft Windows does not work well with Postscript
! so compile for a jpeg output.
#ifdef WIN
      write(outfile,'("hvsratio.jpeg")') 
#else
      write(outfile,'("hvsratio.ps")') 
#endif
      xmin=0.0
      xmax=fmax
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
      open(unit=12,file='hvsratio.gp',access='sequential')
      write(12,*) "# Generated by hvsr.F90  Subroutine hvsrplot()   "
      write(12,*) "# Default is live plot "
      write(12,*) "# for fixed image file remove # on TYPE=image"
      write(12,*) "# and rerun:  gnuplot hvsratio.gp "
#ifdef WIN
      write(12,*) "TYPE=""live""                                    "
      write(12,*) "#TYPE=""image""                                  "
      write(12,*) "if (TYPE ne ""live"" ) {                         "
      write(12,*) "set terminal jpeg large size 500,500 enhanced"
      write(12,*) "set output '\"
      write(12,100) outfile
  100 format(a13,"'")
      write(12,*) " } else {                                      "
      write(12,*)"TRM=GPVAL_TERM                   "
      write(12,*)"if (TRM eq ""unknown"" ) {set terminal x11 1 persist} "
      write(12,*)"if (TRM eq ""qt"") { set terminal wxt 1 persist  } "
      write(12,*)"if (TRM eq ""wxt"") { set terminal wxt 1 persist } "
      write(12,*)"if (TRM eq ""x11"") { set terminal x11 1 persist } "
      write(12,*)" }"
#else
      write(12,*) "TYPE=""live""                                    "
      write(12,*) "#TYPE=""image""                                  "
      write(12,*) "if (TYPE ne ""live"" ) {                         "
      write(12,*) "set terminal postscript size 6.0,3.0 in color \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output '\"
      write(12,100) outfile
  100 format(a11,"'")
      write(12,*) " } else {                                      "
      write(12,*)"TRM=GPVAL_TERM                   "
      write(12,*)"if (TRM eq ""unknown"" ) {set terminal x11 1 persist} "
      write(12,*)"if (TRM eq ""qt"") { set terminal wxt 1 persist  } "
      write(12,*)"if (TRM eq ""wxt"") { set terminal wxt 1 persist } "
      write(12,*)"if (TRM eq ""x11"") { set terminal x11 1 persist } "
      write(12,*)" }"
#endif 
      write(12,*) "set grid                                          "
!      write(12,*) "set nokey" 
      write(12,*) "set xlabel '\"                                    
      write(12,'("Frequency (Hz) \")')
      write(12,*) " ' "
      write(12,*) "set ylabel '\"
      write(12,'("H/V Spectral Ratio\")') 
      write(12,*) " ' "
      write(12,*) "set title 'Horizontal/Vertical Spectral Ratio \"
      write(12,'(" H=ch",I2,"   V=ch",I2,"\")') chH,chV
      write(12,*) " '"
      write(12,'(a)') sxrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2) w l ls 1 lc rgb ""red"" t ""Ratio"", "
      do j=1,npr2h
      write(12,'(2(E14.7,1x))') freq(j),ampHV(j)
      enddo
      write(12,*) "e "
      close(12)
      bz=0         !strings terminate with a zero in C-language
      write(cmd,'(''gnuplot hvsratio.gp'',a)') bz
      call runcmd(cmd)
      return
      end
!#
! --------------------------------------------------------------
! | AUTHOR: P. MICHAELS  http://173.255.241.228                |
! | subroutine: hvsrauto.f                                     |
! |                                                            |
! | copyright (c) 2021 P. Michaels, all rights reserved        |
! | See GNU public license                                     |
! | Plots Horizontal to vertical spectral ratio H/V(freq)      |
! |                                                            |
! |REQUIRES GNUPLOT:                                           |
! |GNUPLOT is GNU plot library which supports C pipes and calls|
! |The program is to write out a *.gp gnuplot file and then    |
! |use runcmd() interface to execute a system call from C.     |
! |See BSU sublibC for the runcmd.c code whose object module is|
! |linked to this Fortran 90 program.                          |
! |                                                            |
! --------------------------------------------------------------
      subroutine hvsrauto(mpts,phi1,phi2,chV,chH)
      real*4  phi1(*),phi2(*)
      character*80 sxrange,syrange
      character*20 outfile
      character*80 cmd
      integer chV,chH
! fill outfile name
! Note: Microsoft Windows does not work well with Postscript
! so compile for a jpeg output.
#ifdef WIN
      write(outfile,'("hvsrauto.jpeg")') 
#else
      write(outfile,'("hvsrauto.ps")') 
#endif
      xmin=0.0
      xmax=mpts
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
      open(unit=12,file='hvsrauto.gp',access='sequential')
      write(12,*) "# Generated by hvsr.F90  Subroutine hvsrauto()   "
      write(12,*) "# Default is live plot "
      write(12,*) "# for fixed image file remove # on TYPE=image"
      write(12,*) "# and rerun:  gnuplot hvsrauto.gp "
#ifdef WIN
      write(12,*) "TYPE=""live""                                    "
      write(12,*) "#TYPE=""image""                                  "
      write(12,*) "if (TYPE ne ""live"" ) {                         "
      write(12,*) "set terminal jpeg large size 500,500 enhanced"
      write(12,*) "set output '\"
      write(12,100) outfile
  100 format(a13,"'")
      write(12,*) " } else {                                      "
      write(12,*)"TRM=GPVAL_TERM                   "
      write(12,*)"if (TRM eq ""unknown"" ) {set terminal x11 2 persist} "
      write(12,*)"if (TRM eq ""qt"") { set terminal wxt 2 persist  } "
      write(12,*)"if (TRM eq ""wxt"") { set terminal wxt 2 persist } "
      write(12,*)"if (TRM eq ""x11"") { set terminal x11 2 persist } "
      write(12,*)" }"
#else
      write(12,*) "TYPE=""live""                                    "
      write(12,*) "#TYPE=""image""                                  "
      write(12,*) "if (TYPE ne ""live"" ) {                         "
      write(12,*) "set terminal postscript size 6.0,3.0 in color \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output '\"
      write(12,100) outfile
  100 format(a11,"'")
      write(12,*) " } else {                                      "
      write(12,*)"TRM=GPVAL_TERM                   "
      write(12,*)"if (TRM eq ""unknown"" ) {set terminal x11 2 persist} "
      write(12,*)"if (TRM eq ""qt"") { set terminal wxt 2 persist  } "
      write(12,*)"if (TRM eq ""wxt"") { set terminal wxt 2 persist } "
      write(12,*)"if (TRM eq ""x11"") { set terminal x11 2 persist } "
      write(12,*)" }"
#endif 
      write(12,*) "set grid                                          "
!      write(12,*) "set nokey" 
      write(12,*) "set xlabel '\"                                    
      write(12,'("Autocorrelation Sample Lag\")')
      write(12,*) " ' "
      write(12,*) "set ylabel '\"
      write(12,'("Amplitude \")') 
      write(12,*) " ' "
      write(12,*) "set title 'Autocorrelations \"
      write(12,'(" H=ch",I2,"   V=ch",I2,"\")') chH,chV
      write(12,*) " '"
      write(12,'(a)') sxrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2) w l ls 1 lc rgb ""red"" t ""Horiz"", \"
      write(12,*) " '' u ($1):($2) w l ls 1 lc rgb ""blue"" t ""Vert""  , "
      do  j=1,mpts
      write(12,'(1x,I5.5,2x,E14.7,1x)') j,phi2(j)
      enddo
      write(12,*) "e "
      do  j=1,mpts
      write(12,'(1x,I5.5,2x,E14.7,1x)') j,phi1(j)
      enddo
      write(12,*) "e "
      close(12)
      bz=0         !strings terminate with a zero in C-language
      write(cmd,'(''gnuplot hvsrauto.gp'',a)') bz
      call runcmd(cmd)
      return
      end
